# Test the support for SSL and sockets

import sys
import unittest
from test import test_support
support = test_support
import asyncore
import socket
import select
import time
import gc
import os
import errno
import pprint
import urllib, urlparse
import traceback
import weakref
import functools
import platform
from contextlib import closing

from BaseHTTPServer import HTTPServer
from SimpleHTTPServer import SimpleHTTPRequestHandler

ssl = test_support.import_module("ssl")

try:
    PROTOCOLS = sorted(ssl._PROTOCOL_NAMES)
except AttributeError:
    PROTOCOLS = ()

HOST = test_support.HOST
CERTFILE = None
SVN_PYTHON_ORG_ROOT_CERT = None

def handle_error(prefix):
    exc_format = ' '.join(traceback.format_exception(*sys.exc_info()))
    if test_support.verbose:
        sys.stdout.write(prefix + exc_format)


class BasicTests(unittest.TestCase):

    def test_sslwrap_simple(self):
        # A crude test for the legacy API
        try:
            ssl.sslwrap_simple(socket.socket(socket.AF_INET))
        except IOError, e:
            if e.errno == 32: # broken pipe when ssl_sock.do_handshake(), this test doesn't care about that
                pass
            else:
                raise
        try:
            ssl.sslwrap_simple(socket.socket(socket.AF_INET)._sock)
        except IOError, e:
            if e.errno == 32: # broken pipe when ssl_sock.do_handshake(), this test doesn't care about that
                pass
            else:
                raise

# Issue #9415: Ubuntu hijacks their OpenSSL and forcefully disables SSLv2
def skip_if_broken_ubuntu_ssl(func):
    if hasattr(ssl, 'PROTOCOL_SSLv2'):
        if hasattr(ssl, 'SSLContext'): # >= 2.7.9
            def f(*args, **kwargs):
                try:
                    ssl.SSLContext(ssl.PROTOCOL_SSLv2)
                except ssl.SSLError:
                    if (ssl.OPENSSL_VERSION_INFO == (0, 9, 8, 15, 15) and
                        platform.linux_distribution() == ('debian', 'squeeze/sid', '')):
                        raise unittest.SkipTest("Patched Ubuntu OpenSSL breaks behaviour")
                return func(*args, **kwargs)
            return f

        # We need to access the lower-level wrapper in order to create an
        # implicit SSL context without trying to connect or listen.
        try:
            import _ssl
        except ImportError:
            # The returned function won't get executed, just ignore the error
            pass

        @functools.wraps(func)
        def f(*args, **kwargs):
            try:
                s = socket.socket(socket.AF_INET)
                _ssl.sslwrap(s._sock, 0, None, None,
                             ssl.CERT_NONE, ssl.PROTOCOL_SSLv2, None, None)
            except ssl.SSLError as e:
                if (ssl.OPENSSL_VERSION_INFO == (0, 9, 8, 15, 15) and
                    platform.linux_distribution() == ('debian', 'squeeze/sid', '')
                    and 'Invalid SSL protocol variant specified' in str(e)):
                    raise unittest.SkipTest("Patched Ubuntu OpenSSL breaks behaviour")
            return func(*args, **kwargs)
        return f
    else:
        return func


class BasicSocketTests(unittest.TestCase):

    def test_constants(self):
        #ssl.PROTOCOL_SSLv2
        ssl.PROTOCOL_SSLv23
        ssl.PROTOCOL_SSLv3
        ssl.PROTOCOL_TLSv1
        ssl.CERT_NONE
        ssl.CERT_OPTIONAL
        ssl.CERT_REQUIRED

    def test_random(self):
        v = ssl.RAND_status()
        if test_support.verbose:
            sys.stdout.write("\n RAND_status is %d (%s)\n"
                             % (v, (v and "sufficient randomness") or
                                "insufficient randomness"))
        try:
            ssl.RAND_egd(1)
        except TypeError:
            pass
        else:
            print "didn't raise TypeError"
        ssl.RAND_add("this is a random string", 75.0)

    def test_parse_cert(self):
        # note that this uses an 'unofficial' function in _ssl.c,
        # provided solely for this test, to exercise the certificate
        # parsing code
        try:
            p = ssl._ssl._test_decode_cert(CERTFILE, False)
        except TypeError: # >= 2.7.9
            p = ssl._ssl._test_decode_cert(CERTFILE)
        if test_support.verbose:
            sys.stdout.write("\n" + pprint.pformat(p) + "\n")
        self.assertEqual(p['subject'],
                         ((('countryName', 'XY'),),
                          (('localityName', 'Castle Anthrax'),),
                          (('organizationName', 'Python Software Foundation'),),
                          (('commonName', 'localhost'),))
                        )
        self.assertEqual(p['subjectAltName'], (('DNS', 'localhost'),))
        # Issue #13034: the subjectAltName in some certificates
        # (notably projects.developer.nokia.com:443) wasn't parsed
        p = ssl._ssl._test_decode_cert(NOKIACERT)
        if test_support.verbose:
            sys.stdout.write("\n" + pprint.pformat(p) + "\n")
        self.assertEqual(p['subjectAltName'],
                         (('DNS', 'projects.developer.nokia.com'),
                          ('DNS', 'projects.forum.nokia.com'))
                        )

    def test_DER_to_PEM(self):
        with open(SVN_PYTHON_ORG_ROOT_CERT, 'r') as f:
            pem = f.read()
        d1 = ssl.PEM_cert_to_DER_cert(pem)
        p2 = ssl.DER_cert_to_PEM_cert(d1)
        d2 = ssl.PEM_cert_to_DER_cert(p2)
        self.assertEqual(d1, d2)
        if not p2.startswith(ssl.PEM_HEADER + '\n'):
            self.fail("DER-to-PEM didn't include correct header:\n%r\n" % p2)
        if not p2.endswith('\n' + ssl.PEM_FOOTER + '\n'):
            self.fail("DER-to-PEM didn't include correct footer:\n%r\n" % p2)

    def test_openssl_version(self):
        n = ssl.OPENSSL_VERSION_NUMBER
        t = ssl.OPENSSL_VERSION_INFO
        s = ssl.OPENSSL_VERSION
        self.assertIsInstance(n, (int, long))
        self.assertIsInstance(t, tuple)
        self.assertIsInstance(s, str)
        # Some sanity checks follow
        # >= 0.9
        self.assertGreaterEqual(n, 0x900000)
        # < 2.0
        self.assertLess(n, 0x20000000)
        major, minor, fix, patch, status = t
        self.assertGreaterEqual(major, 0)
        self.assertLess(major, 2)
        self.assertGreaterEqual(minor, 0)
        self.assertLess(minor, 256)
        self.assertGreaterEqual(fix, 0)
        self.assertLess(fix, 256)
        self.assertGreaterEqual(patch, 0)
        self.assertLessEqual(patch, 26)
        self.assertGreaterEqual(status, 0)
        self.assertLessEqual(status, 15)
        # Version string as returned by OpenSSL, the format might change
        self.assertTrue(s.startswith("OpenSSL {:d}.{:d}.{:d}".format(major, minor, fix)),
                        (s, t))

    def test_ciphers(self):
        if not test_support.is_resource_enabled('network'):
            return
        remote = ("svn.python.org", 443)
        with test_support.transient_internet(remote[0]):
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_NONE, ciphers="ALL")
            s.connect(remote)
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_NONE, ciphers="DEFAULT")
            s.connect(remote)
            # Error checking can happen at instantiation (>=2.7.9) or when connecting
            with self.assertRaisesRegexp(ssl.SSLError, "No cipher can be selected"):
                s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_NONE, ciphers="^$:,;?*'dorothyx")
                s.connect(remote)

    @test_support.cpython_only
    def test_refcycle(self):
        # Issue #7943: an SSL object doesn't create reference cycles with
        # itself.
        s = socket.socket(socket.AF_INET)
        ss = ssl.wrap_socket(s)
        wr = weakref.ref(ss)
        del ss
        self.assertEqual(wr(), None)

    def test_wrapped_unconnected(self):
        # The _delegate_methods in socket.py are correctly delegated to by an
        # unconnected SSLSocket, so they will raise a socket.error rather than
        # something unexpected like TypeError.
        s = socket.socket(socket.AF_INET)
        ss = ssl.wrap_socket(s)
        self.assertRaises(socket.error, ss.recv, 1)
        self.assertRaises(socket.error, ss.recv_into, bytearray(b'x'))
        self.assertRaises(socket.error, ss.recvfrom, 1)
        self.assertRaises(socket.error, ss.recvfrom_into, bytearray(b'x'), 1)
        self.assertRaises(socket.error, ss.send, b'x')
        self.assertRaises(socket.error, ss.sendto, b'x', ('0.0.0.0', 0))


class NetworkedTests(unittest.TestCase):

    def test_connect(self):
        with test_support.transient_internet("svn.python.org"):
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_NONE)
            s.connect(("svn.python.org", 443))
            c = s.getpeercert()
            if c:
                self.fail("Peer cert %s shouldn't be here!")
            s.close()

            # this should fail because we have no verification certs
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_REQUIRED)
            try:
                s.connect(("svn.python.org", 443))
            except ssl.SSLError:
                pass
            finally:
                s.close()

            # this should succeed because we specify the root cert
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_REQUIRED,
                                ca_certs=SVN_PYTHON_ORG_ROOT_CERT)
            try:
                s.connect(("svn.python.org", 443))
            finally:
                s.close()

    def test_connect_ex(self):
        # Issue #11326: check connect_ex() implementation
        with test_support.transient_internet("svn.python.org"):
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_REQUIRED,
                                ca_certs=SVN_PYTHON_ORG_ROOT_CERT)
            try:
                self.assertEqual(0, s.connect_ex(("svn.python.org", 443)))
                self.assertTrue(s.getpeercert())
            finally:
                s.close()

    def test_non_blocking_connect_ex(self):
        # Issue #11326: non-blocking connect_ex() should allow handshake
        # to proceed after the socket gets ready.
        with test_support.transient_internet("svn.python.org"):
            s = ssl.wrap_socket(socket.socket(socket.AF_INET),
                                cert_reqs=ssl.CERT_REQUIRED,
                                ca_certs=SVN_PYTHON_ORG_ROOT_CERT,
                                do_handshake_on_connect=False)
            try:
                s.setblocking(False)
                rc = s.connect_ex(('svn.python.org', 443))
                # EWOULDBLOCK under Windows, EINPROGRESS elsewhere
                self.assertIn(rc, (0, errno.EINPROGRESS, errno.EWOULDBLOCK))
                # Wait for connect to finish
                select.select([], [s], [], 5.0)
                # Non-blocking handshake
                while True:
                    try:
                        s.do_handshake()
                        break
                    except ssl.SSLError as err:
                        if err.args[0] == ssl.SSL_ERROR_WANT_READ:
                            select.select([s], [], [], 5.0)
                        elif err.args[0] == ssl.SSL_ERROR_WANT_WRITE:
                            select.select([], [s], [], 5.0)
                        else:
                            raise
                # SSL established
                self.assertTrue(s.getpeercert())
            finally:
                s.close()

    @unittest.skipIf(os.name == "nt", "Can't use a socket as a file under Windows")
    def test_makefile_close(self):
        # Issue #5238: creating a file-like object with makefile() shouldn't
        # delay closing the underlying "real socket" (here tested with its
        # file descriptor, hence skipping the test under Windows).
        with test_support.transient_internet("svn.python.org"):
            ss = ssl.wrap_socket(socket.socket(socket.AF_INET))
            ss.connect(("svn.python.org", 443))
            fd = ss.fileno()
            f = ss.makefile()
            f.close()
            # The fd is still open
            os.read(fd, 0)
            # Closing the SSL socket should close the fd too
            ss.close()
            gc.collect()
            with self.assertRaises(OSError) as e:
                os.read(fd, 0)
            self.assertEqual(e.exception.errno, errno.EBADF)

    def test_non_blocking_handshake(self):
        with test_support.transient_internet("svn.python.org"):
            s = socket.socket(socket.AF_INET)
            s.connect(("svn.python.org", 443))
            s.setblocking(False)
            s = ssl.wrap_socket(s,
                                cert_reqs=ssl.CERT_NONE,
                                do_handshake_on_connect=False)
            count = 0
            while True:
                try:
                    count += 1
                    s.do_handshake()
                    break
                except ssl.SSLError, err:
                    if err.args[0] == ssl.SSL_ERROR_WANT_READ:
                        select.select([s], [], [])
                    elif err.args[0] == ssl.SSL_ERROR_WANT_WRITE:
                        select.select([], [s], [])
                    else:
                        raise
            s.close()
            if test_support.verbose:
                sys.stdout.write("\nNeeded %d calls to do_handshake() to establish session.\n" % count)

    def test_get_server_certificate(self):
        with test_support.transient_internet("svn.python.org"):
            pem = ssl.get_server_certificate(("svn.python.org", 443))
            if not pem:
                self.fail("No server certificate on svn.python.org:443!")

            try:
                pem = ssl.get_server_certificate(("svn.python.org", 443), ca_certs=CERTFILE)
            except ssl.SSLError:
                #should fail
                pass
            else:
                self.fail("Got server certificate %s for svn.python.org!" % pem)

            pem = ssl.get_server_certificate(("svn.python.org", 443), ca_certs=SVN_PYTHON_ORG_ROOT_CERT)
            if not pem:
                self.fail("No server certificate on svn.python.org:443!")
            if test_support.verbose:
                sys.stdout.write("\nVerified certificate for svn.python.org:443 is\n%s\n" % pem)

    def test_algorithms(self):
        # Issue #8484: all algorithms should be available when verifying a
        # certificate.
        # SHA256 was added in OpenSSL 0.9.8
        if ssl.OPENSSL_VERSION_INFO < (0, 9, 8, 0, 15):
            self.skipTest("SHA256 not available on %r" % ssl.OPENSSL_VERSION)
        if not hasattr(ssl, 'SSLContext') or not getattr(ssl, 'HAS_SNI', True):
            # The Python 2.7.8 test lib doesn't run this
            self.skipTest('remote host needs SNI, only available on Python 3.2+')
        # NOTE: https://sha256.tbs-internet.com is another possible test host
        remote = ("sha256.tbs-internet.com", 443)
        sha256_cert = os.path.join(os.path.dirname(__file__), "sha256.pem")

        with test_support.transient_internet("sha256.tbs-internet.com"):
            ctx = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
            ctx.verify_mode = ssl.CERT_REQUIRED
            ctx.load_verify_locations(sha256_cert)
            s = ctx.wrap_socket(socket.socket(socket.AF_INET),
                                server_hostname="sha256.tbs-internet.com")
            try:
                s.connect(remote)
                if test_support.verbose:
                    sys.stdout.write("\nCipher with %r is %r\n" %
                                     (remote, s.cipher()))
                    sys.stdout.write("Certificate is:\n%s\n" %
                                     pprint.pformat(s.getpeercert()))
            finally:
                s.close()


try:
    import threading
except ImportError:
    _have_threads = False
else:
    _have_threads = True

    class ThreadedEchoServer(threading.Thread):

        class ConnectionHandler(threading.Thread):

            """A mildly complicated class, because we want it to work both
            with and without the SSL wrapper around the socket connection, so
            that we can test the STARTTLS functionality."""

            def __init__(self, server, connsock):
                self.server = server
                self.running = False
                self.sock = connsock
                self.sock.setblocking(1)
                self.sslconn = None
                threading.Thread.__init__(self)
                self.daemon = True

            def show_conn_details(self):
                if self.server.certreqs == ssl.CERT_REQUIRED:
                    cert = self.sslconn.getpeercert()
                    if test_support.verbose and self.server.chatty:
                        sys.stdout.write(" client cert is " + pprint.pformat(cert) + "\n")
                    cert_binary = self.sslconn.getpeercert(True)
                    if test_support.verbose and self.server.chatty:
                        sys.stdout.write(" cert binary is " + str(len(cert_binary)) + " bytes\n")
                cipher = self.sslconn.cipher()
                if test_support.verbose and self.server.chatty:
                    sys.stdout.write(" server: connection cipher is now " + str(cipher) + "\n")

            if hasattr(ssl, 'SSLContext'): # >= 2.7.9
                def wrap_conn(self):
                    try:
                        self.sslconn = self.server.context.wrap_socket(
                            self.sock, server_side=True)
                        self.server.selected_protocols.append(self.sslconn.selected_npn_protocol())
                    except socket.error as e:
                        # We treat ConnectionResetError as though it were an
                        # SSLError - OpenSSL on Ubuntu abruptly closes the
                        # connection when asked to use an unsupported protocol.
                        #
                        # XXX Various errors can have happened here, for example
                        # a mismatching protocol version, an invalid certificate,
                        # or a low-level bug. This should be made more discriminating.
                        if not isinstance(e, ssl.SSLError) and e.errno != errno.ECONNRESET:
                            raise
                        self.server.conn_errors.append(e)
                        if self.server.chatty:
                            handle_error("\n server:  bad connection attempt from " + repr(self.addr) + ":\n")
                        self.running = False
                        self.server.stop()
                        self.close()
                        return False
                    else:
                        if self.server.context.verify_mode == ssl.CERT_REQUIRED:
                            cert = self.sslconn.getpeercert()
                            if support.verbose and self.server.chatty:
                                sys.stdout.write(" client cert is " + pprint.pformat(cert) + "\n")
                            cert_binary = self.sslconn.getpeercert(True)
                            if support.verbose and self.server.chatty:
                                sys.stdout.write(" cert binary is " + str(len(cert_binary)) + " bytes\n")
                        cipher = self.sslconn.cipher()
                        if support.verbose and self.server.chatty:
                            sys.stdout.write(" server: connection cipher is now " + str(cipher) + "\n")
                            sys.stdout.write(" server: selected protocol is now "
                                    + str(self.sslconn.selected_npn_protocol()) + "\n")
                        return True
            else:
                def wrap_conn(self):
                    try:
                        self.sslconn = ssl.wrap_socket(self.sock, server_side=True,
                                                       certfile=self.server.certificate,
                                                       ssl_version=self.server.protocol,
                                                       ca_certs=self.server.cacerts,
                                                       cert_reqs=self.server.certreqs,
                                                       ciphers=self.server.ciphers)
                    except ssl.SSLError as e:
                        # XXX Various errors can have happened here, for example
                        # a mismatching protocol version, an invalid certificate,
                        # or a low-level bug. This should be made more discriminating.
                        self.server.conn_errors.append(e)
                        if self.server.chatty:
                            handle_error("\n server:  bad connection attempt from " +
                                         str(self.sock.getpeername()) + ":\n")
                        self.close()
                        self.running = False
                        self.server.stop()
                        return False
                    else:
                        return True

            def read(self):
                if self.sslconn:
                    return self.sslconn.read()
                else:
                    return self.sock.recv(1024)

            def write(self, bytes):
                if self.sslconn:
                    return self.sslconn.write(bytes)
                else:
                    return self.sock.send(bytes)

            def close(self):
                if self.sslconn:
                    self.sslconn.close()
                else:
                    self.sock._sock.close()

            def run(self):
                self.running = True
                if not self.server.starttls_server:
                    if isinstance(self.sock, ssl.SSLSocket):
                        self.sslconn = self.sock
                    elif not self.wrap_conn():
                        return

                while self.running:
                    try:
                        msg = self.read()
                        if not msg:
                            # eof, so quit this handler
                            self.running = False
                            self.close()
                        elif msg.strip() == 'over':
                            if test_support.verbose and self.server.connectionchatty:
                                sys.stdout.write(" server: client closed connection\n")
                            self.close()
                            return
                        elif self.server.starttls_server and msg.strip() == 'STARTTLS':
                            if test_support.verbose and self.server.connectionchatty:
                                sys.stdout.write(" server: read STARTTLS from client, sending OK...\n")
                            self.write("OK\n")
                            if not self.wrap_conn():
                                return
                        elif self.server.starttls_server and self.sslconn and msg.strip() == 'ENDTLS':
                            if test_support.verbose and self.server.connectionchatty:
                                sys.stdout.write(" server: read ENDTLS from client, sending OK...\n")
                            self.write("OK\n")
                            self.sslconn.unwrap()
                            self.sslconn = None
                            if test_support.verbose and self.server.connectionchatty:
                                sys.stdout.write(" server: connection is now unencrypted...\n")
                        else:
                            if (test_support.verbose and
                                self.server.connectionchatty):
                                ctype = (self.sslconn and "encrypted") or "unencrypted"
                                sys.stdout.write(" server: read %s (%s), sending back %s (%s)...\n"
                                                 % (repr(msg), ctype, repr(msg.lower()), ctype))
                            self.write(msg.lower())
                    except ssl.SSLError:
                        if self.server.chatty:
                            handle_error("Test server failure:\n")
                        self.close()
                        self.running = False
                        # normally, we'd just stop here, but for the test
                        # harness, we want to stop the server
                        self.server.stop()

        if hasattr(ssl, 'SSLContext'): # >= 2.7.9
            def __init__(self, certificate=None, ssl_version=None,
                         certreqs=None, cacerts=None,
                         chatty=True, connectionchatty=False, starttls_server=False,
                         npn_protocols=None, ciphers=None, context=None):
                if context:
                    self.context = context
                else:
                    self.context = ssl.SSLContext(ssl_version
                                                  if ssl_version is not None
                                                  else ssl.PROTOCOL_TLSv1)
                    self.context.verify_mode = (certreqs if certreqs is not None
                                                else ssl.CERT_NONE)
                    if cacerts:
                        self.context.load_verify_locations(cacerts)
                    if certificate:
                        self.context.load_cert_chain(certificate)
                    if npn_protocols:
                        self.context.set_npn_protocols(npn_protocols)
                    if ciphers:
                        self.context.set_ciphers(ciphers)
                self.chatty = chatty
                self.connectionchatty = connectionchatty
                self.starttls_server = starttls_server
                self.sock = socket.socket()
                self.port = support.bind_port(self.sock)
                self.flag = None
                self.active = False
                self.selected_protocols = []
                self.conn_errors = []
                threading.Thread.__init__(self)
                self.daemon = True
        else:
            def __init__(self, certificate, ssl_version=None,
                         certreqs=None, cacerts=None,
                         chatty=True, connectionchatty=False, starttls_server=False,
                         wrap_accepting_socket=False, ciphers=None):

                if ssl_version is None:
                    ssl_version = ssl.PROTOCOL_TLSv1
                if certreqs is None:
                    certreqs = ssl.CERT_NONE
                self.certificate = certificate
                self.protocol = ssl_version
                self.certreqs = certreqs
                self.cacerts = cacerts
                self.ciphers = ciphers
                self.chatty = chatty
                self.connectionchatty = connectionchatty
                self.starttls_server = starttls_server
                self.sock = socket.socket()
                self.flag = None
                if wrap_accepting_socket:
                    self.sock = ssl.wrap_socket(self.sock, server_side=True,
                                                certfile=self.certificate,
                                                cert_reqs = self.certreqs,
                                                ca_certs = self.cacerts,
                                                ssl_version = self.protocol,
                                                ciphers = self.ciphers)
                    if test_support.verbose and self.chatty:
                        sys.stdout.write(' server:  wrapped server socket as %s\n' % str(self.sock))
                self.port = test_support.bind_port(self.sock)
                self.active = False
                self.conn_errors = []
                threading.Thread.__init__(self)
                self.daemon = True

        def __enter__(self):
            self.start(threading.Event())
            self.flag.wait()
            return self

        def __exit__(self, *args):
            self.stop()
            self.join()

        def start(self, flag=None):
            self.flag = flag
            threading.Thread.start(self)

        def run(self):
            self.sock.settimeout(0.05)
            self.sock.listen(5)
            self.active = True
            if self.flag:
                # signal an event
                self.flag.set()
            while self.active:
                try:
                    newconn, connaddr = self.sock.accept()
                    if test_support.verbose and self.chatty:
                        sys.stdout.write(' server:  new connection from '
                                         + str(connaddr) + '\n')
                    handler = self.ConnectionHandler(self, newconn)
                    handler.start()
                    handler.join()
                except socket.timeout:
                    pass
                except KeyboardInterrupt:
                    self.stop()
            self.sock.close()

        def stop(self):
            self.active = False

    class AsyncoreEchoServer(threading.Thread):

        class EchoServer(asyncore.dispatcher):

            class ConnectionHandler(asyncore.dispatcher_with_send):

                def __init__(self, conn, certfile):
                    asyncore.dispatcher_with_send.__init__(self, conn)
                    self.socket = ssl.wrap_socket(conn, server_side=True,
                                                  certfile=certfile,
                                                  do_handshake_on_connect=False)
                    self._ssl_accepting = True

                def readable(self):
                    if isinstance(self.socket, ssl.SSLSocket):
                        while self.socket.pending() > 0:
                            self.handle_read_event()
                    return True

                def _do_ssl_handshake(self):
                    try:
                        self.socket.do_handshake()
                    except ssl.SSLError, err:
                        if err.args[0] in (ssl.SSL_ERROR_WANT_READ,
                                           ssl.SSL_ERROR_WANT_WRITE):
                            return
                        elif err.args[0] == ssl.SSL_ERROR_EOF:
                            return self.handle_close()
                        raise
                    except socket.error, err:
                        if err.args[0] == errno.ECONNABORTED:
                            return self.handle_close()
                    else:
                        self._ssl_accepting = False

                def handle_read(self):
                    if self._ssl_accepting:
                        self._do_ssl_handshake()
                    else:
                        data = self.recv(1024)
                        if data and data.strip() != 'over':
                            self.send(data.lower())

                def handle_close(self):
                    self.close()
                    if test_support.verbose:
                        sys.stdout.write(" server:  closed connection %s\n" % self.socket)

                def handle_error(self):
                    raise

            def __init__(self, certfile):
                self.certfile = certfile
                asyncore.dispatcher.__init__(self)
                self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
                self.port = test_support.bind_port(self.socket)
                self.listen(5)

            def handle_accept(self):
                sock_obj, addr = self.accept()
                if test_support.verbose:
                    sys.stdout.write(" server:  new connection from %s:%s\n" %addr)
                self.ConnectionHandler(sock_obj, self.certfile)

            def handle_error(self):
                raise

        def __init__(self, certfile):
            self.flag = None
            self.active = False
            self.server = self.EchoServer(certfile)
            self.port = self.server.port
            threading.Thread.__init__(self)
            self.daemon = True

        def __str__(self):
            return "<%s %s>" % (self.__class__.__name__, self.server)

        def __enter__(self):
            self.start(threading.Event())
            self.flag.wait()
            return self

        def __exit__(self, *args):
            if test_support.verbose:
                sys.stdout.write(" cleanup: stopping server.\n")
            self.stop()
            if test_support.verbose:
                sys.stdout.write(" cleanup: joining server thread.\n")
            self.join()
            if test_support.verbose:
                sys.stdout.write(" cleanup: successfully joined.\n")

        def start(self, flag=None):
            self.flag = flag
            threading.Thread.start(self)

        def run(self):
            self.active = True
            if self.flag:
                self.flag.set()
            while self.active:
                asyncore.loop(0.05)

        def stop(self):
            self.active = False
            self.server.close()

    class SocketServerHTTPSServer(threading.Thread):

        class HTTPSServer(HTTPServer):

            def __init__(self, server_address, RequestHandlerClass, certfile):
                HTTPServer.__init__(self, server_address, RequestHandlerClass)
                # we assume the certfile contains both private key and certificate
                self.certfile = certfile
                self.allow_reuse_address = True

            def __str__(self):
                return ('<%s %s:%s>' %
                        (self.__class__.__name__,
                         self.server_name,
                         self.server_port))

            def get_request(self):
                # override this to wrap socket with SSL
                sock, addr = self.socket.accept()
                sslconn = ssl.wrap_socket(sock, server_side=True,
                                          certfile=self.certfile)
                return sslconn, addr

        class RootedHTTPRequestHandler(SimpleHTTPRequestHandler):
            # need to override translate_path to get a known root,
            # instead of using os.curdir, since the test could be
            # run from anywhere

            server_version = "TestHTTPS/1.0"

            root = None

            def translate_path(self, path):
                """Translate a /-separated PATH to the local filename syntax.

                Components that mean special things to the local file system
                (e.g. drive or directory names) are ignored.  (XXX They should
                probably be diagnosed.)

                """
                # abandon query parameters
                path = urlparse.urlparse(path)[2]
                path = os.path.normpath(urllib.unquote(path))
                words = path.split('/')
                words = filter(None, words)
                path = self.root
                for word in words:
                    drive, word = os.path.splitdrive(word)
                    head, word = os.path.split(word)
                    if word in self.root: continue
                    path = os.path.join(path, word)
                return path

            def log_message(self, format, *args):

                # we override this to suppress logging unless "verbose"

                if test_support.verbose:
                    sys.stdout.write(" server (%s:%d %s):\n   [%s] %s\n" %
                                     (self.server.server_address,
                                      self.server.server_port,
                                      self.request.cipher(),
                                      self.log_date_time_string(),
                                      format%args))


        def __init__(self, certfile):
            self.flag = None
            self.RootedHTTPRequestHandler.root = os.path.split(CERTFILE)[0]
            self.server = self.HTTPSServer(
                (HOST, 0), self.RootedHTTPRequestHandler, certfile)
            self.port = self.server.server_port
            threading.Thread.__init__(self)
            self.daemon = True

        def __str__(self):
            return "<%s %s>" % (self.__class__.__name__, self.server)

        def start(self, flag=None):
            self.flag = flag
            threading.Thread.start(self)

        def run(self):
            if self.flag:
                self.flag.set()
            self.server.serve_forever(0.05)

        def stop(self):
            self.server.shutdown()


    def bad_cert_test(certfile):
        """
        Launch a server with CERT_REQUIRED, and check that trying to
        connect to it with the given client certificate fails.
        """
        server = ThreadedEchoServer(CERTFILE,
                                    certreqs=ssl.CERT_REQUIRED,
                                    cacerts=CERTFILE, chatty=False)
        with server:
            try:
                s = ssl.wrap_socket(socket.socket(),
                                    certfile=certfile,
                                    ssl_version=ssl.PROTOCOL_TLSv1)
                s.connect((HOST, server.port))
            except ssl.SSLError, x:
                if test_support.verbose:
                    sys.stdout.write("\nSSLError is %s\n" % x[1])
            except socket.error, x:
                if test_support.verbose:
                    sys.stdout.write("\nsocket.error is %s\n" % x[1])
            else:
                raise AssertionError("Use of invalid cert should have failed!")

    if hasattr(ssl, 'SSLContext'): # >= 2.7.9
        def server_params_test(client_context, server_context, indata=b"FOO\n",
                           chatty=True, connectionchatty=False, sni_name=None):
            """
            Launch a server, connect a client to it and try various reads
            and writes.
            """
            stats = {}
            server = ThreadedEchoServer(context=server_context,
                                        chatty=chatty,
                                        connectionchatty=False)
            with server:
                with closing(client_context.wrap_socket(socket.socket(),
                        server_hostname=sni_name)) as s:
                    s.connect((HOST, server.port))
                    for arg in [indata, bytearray(indata), memoryview(indata)]:
                        if connectionchatty:
                            if support.verbose:
                                sys.stdout.write(
                                    " client:  sending %r...\n" % indata)
                        s.write(arg)
                        outdata = s.read()
                        if connectionchatty:
                            if support.verbose:
                                sys.stdout.write(" client:  read %r\n" % outdata)
                        if outdata != indata.lower():
                            raise AssertionError(
                                "bad data <<%r>> (%d) received; expected <<%r>> (%d)\n"
                                % (outdata[:20], len(outdata),
                                   indata[:20].lower(), len(indata)))
                    s.write(b"over\n")
                    if connectionchatty:
                        if support.verbose:
                            sys.stdout.write(" client:  closing connection.\n")
                    stats.update({
                        'compression': s.compression(),
                        'cipher': s.cipher(),
                        'peercert': s.getpeercert(),
                        'client_npn_protocol': s.selected_npn_protocol(),
                        'version': s.version(),
                    })
                    s.close()
                stats['server_npn_protocols'] = server.selected_protocols
            return stats

        def try_protocol_combo(server_protocol, client_protocol, expect_success,
                               certsreqs=None, server_options=0, client_options=0):
            """
            Try to SSL-connect using *client_protocol* to *server_protocol*.
            If *expect_success* is true, assert that the connection succeeds,
            if it's false, assert that the connection fails.
            Also, if *expect_success* is a string, assert that it is the protocol
            version actually used by the connection.
            """
            if certsreqs is None:
                certsreqs = ssl.CERT_NONE
            certtype = {
                ssl.CERT_NONE: "CERT_NONE",
                ssl.CERT_OPTIONAL: "CERT_OPTIONAL",
                ssl.CERT_REQUIRED: "CERT_REQUIRED",
            }[certsreqs]
            if support.verbose:
                formatstr = (expect_success and " %s->%s %s\n") or " {%s->%s} %s\n"
                sys.stdout.write(formatstr %
                                 (ssl.get_protocol_name(client_protocol),
                                  ssl.get_protocol_name(server_protocol),
                                  certtype))
            client_context = ssl.SSLContext(client_protocol)
            client_context.options |= client_options
            server_context = ssl.SSLContext(server_protocol)
            server_context.options |= server_options

            # NOTE: we must enable "ALL" ciphers on the client, otherwise an
            # SSLv23 client will send an SSLv3 hello (rather than SSLv2)
            # starting from OpenSSL 1.0.0 (see issue #8322).
            if client_context.protocol == ssl.PROTOCOL_SSLv23:
                client_context.set_ciphers("ALL")

            for ctx in (client_context, server_context):
                ctx.verify_mode = certsreqs
                ctx.load_cert_chain(CERTFILE)
                ctx.load_verify_locations(CERTFILE)
            try:
                stats = server_params_test(client_context, server_context,
                                           chatty=False, connectionchatty=False)
            # Protocol mismatch can result in either an SSLError, or a
            # "Connection reset by peer" error.
            except ssl.SSLError:
                if expect_success:
                    raise
            except socket.error as e:
                if expect_success or e.errno != errno.ECONNRESET:
                    raise
            else:
                if not expect_success:
                    raise AssertionError(
                        "Client protocol %s succeeded with server protocol %s!"
                        % (ssl.get_protocol_name(client_protocol),
                           ssl.get_protocol_name(server_protocol)))
                elif (expect_success is not True
                      and expect_success != stats['version']):
                    raise AssertionError("version mismatch: expected %r, got %r"
                                         % (expect_success, stats['version']))

    else:
        def server_params_test(certfile, protocol, certreqs, cacertsfile,
                               client_certfile, client_protocol=None, indata="FOO\n",
                               ciphers=None, chatty=True, connectionchatty=False,
                               wrap_accepting_socket=False):
            """
            Launch a server, connect a client to it and try various reads
            and writes.
            """
            server = ThreadedEchoServer(certfile,
                                        certreqs=certreqs,
                                        ssl_version=protocol,
                                        cacerts=cacertsfile,
                                        ciphers=ciphers,
                                        chatty=chatty,
                                        connectionchatty=connectionchatty,
                                        wrap_accepting_socket=wrap_accepting_socket)
            with server:
                # try to connect
                if client_protocol is None:
                    client_protocol = protocol
                s = ssl.wrap_socket(socket.socket(),
                                    certfile=client_certfile,
                                    ca_certs=cacertsfile,
                                    ciphers=ciphers,
                                    cert_reqs=certreqs,
                                    ssl_version=client_protocol)
                s.connect((HOST, server.port))
                for arg in [indata, bytearray(indata), memoryview(indata)]:
                    if connectionchatty:
                        if test_support.verbose:
                            sys.stdout.write(
                                " client:  sending %s...\n" % (repr(arg)))
                    s.write(arg)
                    outdata = s.read()
                    if connectionchatty:
                        if test_support.verbose:
                            sys.stdout.write(" client:  read %s\n" % repr(outdata))
                    if outdata != indata.lower():
                        raise AssertionError(
                            "bad data <<%s>> (%d) received; expected <<%s>> (%d)\n"
                            % (outdata[:min(len(outdata),20)], len(outdata),
                               indata[:min(len(indata),20)].lower(), len(indata)))
                s.write("over\n")
                if connectionchatty:
                    if test_support.verbose:
                        sys.stdout.write(" client:  closing connection.\n")
                s.close()

        def try_protocol_combo(server_protocol,
                               client_protocol,
                               expect_success,
                               certsreqs=None):
            if certsreqs is None:
                certsreqs = ssl.CERT_NONE
            certtype = {
                ssl.CERT_NONE: "CERT_NONE",
                ssl.CERT_OPTIONAL: "CERT_OPTIONAL",
                ssl.CERT_REQUIRED: "CERT_REQUIRED",
            }[certsreqs]
            if test_support.verbose:
                formatstr = (expect_success and " %s->%s %s\n") or " {%s->%s} %s\n"
                sys.stdout.write(formatstr %
                                 (ssl.get_protocol_name(client_protocol),
                                  ssl.get_protocol_name(server_protocol),
                                  certtype))
            try:
                # NOTE: we must enable "ALL" ciphers, otherwise an SSLv23 client
                # will send an SSLv3 hello (rather than SSLv2) starting from
                # OpenSSL 1.0.0 (see issue #8322).
                server_params_test(CERTFILE, server_protocol, certsreqs,
                                   CERTFILE, CERTFILE, client_protocol,
                                   ciphers="ALL", chatty=False)
            # Protocol mismatch can result in either an SSLError, or a
            # "Connection reset by peer" error.
            except ssl.SSLError:
                if expect_success:
                    raise
            except socket.error as e:
                if expect_success or e.errno != errno.ECONNRESET:
                    raise
            else:
                if not expect_success:
                    raise AssertionError(
                        "Client protocol %s succeeded with server protocol %s!"
                        % (ssl.get_protocol_name(client_protocol),
                           ssl.get_protocol_name(server_protocol)))


    class ThreadedTests(unittest.TestCase):

        def test_rude_shutdown(self):
            """A brutal shutdown of an SSL server should raise an IOError
            in the client when attempting handshake.
            """
            listener_ready = threading.Event()
            listener_gone = threading.Event()

            s = socket.socket()
            port = test_support.bind_port(s, HOST)

            # `listener` runs in a thread.  It sits in an accept() until
            # the main thread connects.  Then it rudely closes the socket,
            # and sets Event `listener_gone` to let the main thread know
            # the socket is gone.
            def listener():
                s.listen(5)
                listener_ready.set()
                s.accept()
                s.close()
                listener_gone.set()

            def connector():
                listener_ready.wait()
                c = socket.socket()
                c.connect((HOST, port))
                listener_gone.wait()
                try:
                    ssl_sock = ssl.wrap_socket(c)
                except IOError:
                    pass
                else:
                    self.fail('connecting to closed SSL socket should have failed')

            t = threading.Thread(target=listener)
            t.start()
            try:
                connector()
            finally:
                t.join()

        @skip_if_broken_ubuntu_ssl
        def test_echo(self):
            """Basic test of an SSL client connecting to a server"""
            if test_support.verbose:
                sys.stdout.write("\n")
            if hasattr(ssl, 'SSLContext'): # >= 2.7.9
                for protocol in PROTOCOLS:
                    context = ssl.SSLContext(protocol)
                    context.load_cert_chain(CERTFILE)
                    server_params_test(context, context,
                                       chatty=True, connectionchatty=True)
            else:
                server_params_test(CERTFILE, ssl.PROTOCOL_TLSv1, ssl.CERT_NONE,
                                   CERTFILE, CERTFILE, ssl.PROTOCOL_TLSv1,
                                   chatty=True, connectionchatty=True)

        def test_getpeercert(self):
            if test_support.verbose:
                sys.stdout.write("\n")
            s2 = socket.socket()
            server = ThreadedEchoServer(CERTFILE,
                                        certreqs=ssl.CERT_NONE,
                                        ssl_version=ssl.PROTOCOL_SSLv23,
                                        cacerts=CERTFILE,
                                        chatty=False)
            with server:
                s = ssl.wrap_socket(socket.socket(),
                                    certfile=CERTFILE,
                                    ca_certs=CERTFILE,
                                    cert_reqs=ssl.CERT_REQUIRED,
                                    ssl_version=ssl.PROTOCOL_SSLv23)
                s.connect((HOST, server.port))
                cert = s.getpeercert()
                self.assertTrue(cert, "Can't get peer certificate.")
                cipher = s.cipher()
                if test_support.verbose:
                    sys.stdout.write(pprint.pformat(cert) + '\n')
                    sys.stdout.write("Connection cipher is " + str(cipher) + '.\n')
                if 'subject' not in cert:
                    self.fail("No subject field in certificate: %s." %
                              pprint.pformat(cert))
                if ((('organizationName', 'Python Software Foundation'),)
                    not in cert['subject']):
                    self.fail(
                        "Missing or invalid 'organizationName' field in certificate subject; "
                        "should be 'Python Software Foundation'.")
                s.close()

        def test_empty_cert(self):
            """Connecting with an empty cert file"""
            bad_cert_test(os.path.join(os.path.dirname(__file__) or os.curdir,
                                      "nullcert.pem"))
        def test_malformed_cert(self):
            """Connecting with a badly formatted certificate (syntax error)"""
            bad_cert_test(os.path.join(os.path.dirname(__file__) or os.curdir,
                                       "badcert.pem"))
        def test_nonexisting_cert(self):
            """Connecting with a non-existing cert file"""
            bad_cert_test(os.path.join(os.path.dirname(__file__) or os.curdir,
                                       "wrongcert.pem"))
        def test_malformed_key(self):
            """Connecting with a badly formatted key (syntax error)"""
            bad_cert_test(os.path.join(os.path.dirname(__file__) or os.curdir,
                                       "badkey.pem"))

        @skip_if_broken_ubuntu_ssl
        def test_protocol_sslv2(self):
            """Connecting to an SSLv2 server with various client options"""
            if test_support.verbose:
                sys.stdout.write("\n")
            if not hasattr(ssl, 'PROTOCOL_SSLv2'):
                self.skipTest("PROTOCOL_SSLv2 needed")
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_SSLv2, True)
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_SSLv2, True, ssl.CERT_OPTIONAL)
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_SSLv2, True, ssl.CERT_REQUIRED)
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_SSLv23, False)
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_SSLv3, False)
            try_protocol_combo(ssl.PROTOCOL_SSLv2, ssl.PROTOCOL_TLSv1, False)

        @skip_if_broken_ubuntu_ssl
        def test_protocol_sslv23(self):
            """Connecting to an SSLv23 server with various client options"""
            if test_support.verbose:
                sys.stdout.write("\n")
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv3, True)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv23, True)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_TLSv1, True)

            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv3, True, ssl.CERT_OPTIONAL)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv23, True, ssl.CERT_OPTIONAL)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_TLSv1, True, ssl.CERT_OPTIONAL)

            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv3, True, ssl.CERT_REQUIRED)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_SSLv23, True, ssl.CERT_REQUIRED)
            try_protocol_combo(ssl.PROTOCOL_SSLv23, ssl.PROTOCOL_TLSv1, True, ssl.CERT_REQUIRED)

        @skip_if_broken_ubuntu_ssl
        def test_protocol_sslv3(self):
            """Connecting to an SSLv3 server with various client options"""
            if test_support.verbose:
                sys.stdout.write("\n")
            try_protocol_combo(ssl.PROTOCOL_SSLv3, ssl.PROTOCOL_SSLv3, True)
            try_protocol_combo(ssl.PROTOCOL_SSLv3, ssl.PROTOCOL_SSLv3, True, ssl.CERT_OPTIONAL)
            try_protocol_combo(ssl.PROTOCOL_SSLv3, ssl.PROTOCOL_SSLv3, True, ssl.CERT_REQUIRED)
            if hasattr(ssl, 'PROTOCOL_SSLv2'):
                try_protocol_combo(ssl.PROTOCOL_SSLv3, ssl.PROTOCOL_SSLv2, False)
            try_protocol_combo(ssl.PROTOCOL_SSLv3, ssl.PROTOCOL_TLSv1, False)

        @skip_if_broken_ubuntu_ssl
        def test_protocol_tlsv1(self):
            """Connecting to a TLSv1 server with various client options"""
            if test_support.verbose:
                sys.stdout.write("\n")
            try_protocol_combo(ssl.PROTOCOL_TLSv1, ssl.PROTOCOL_TLSv1, True)
            try_protocol_combo(ssl.PROTOCOL_TLSv1, ssl.PROTOCOL_TLSv1, True, ssl.CERT_OPTIONAL)
            try_protocol_combo(ssl.PROTOCOL_TLSv1, ssl.PROTOCOL_TLSv1, True, ssl.CERT_REQUIRED)
            if hasattr(ssl, 'PROTOCOL_SSLv2'):
                try_protocol_combo(ssl.PROTOCOL_TLSv1, ssl.PROTOCOL_SSLv2, False)
            try_protocol_combo(ssl.PROTOCOL_TLSv1, ssl.PROTOCOL_SSLv3, False)

        def test_starttls(self):
            """Switching from clear text to encrypted and back again."""
            msgs = ("msg 1", "MSG 2", "STARTTLS", "MSG 3", "msg 4", "ENDTLS", "msg 5", "msg 6")

            server = ThreadedEchoServer(CERTFILE,
                                        ssl_version=ssl.PROTOCOL_TLSv1,
                                        starttls_server=True,
                                        chatty=True,
                                        connectionchatty=True)
            wrapped = False
            with server:
                s = socket.socket()
                s.setblocking(1)
                s.connect((HOST, server.port))
                if test_support.verbose:
                    sys.stdout.write("\n")
                for indata in msgs:
                    if test_support.verbose:
                        sys.stdout.write(
                            " client:  sending %s...\n" % repr(indata))
                    if wrapped:
                        conn.write(indata)
                        outdata = conn.read()
                    else:
                        s.send(indata)
                        outdata = s.recv(1024)
                    if (indata == "STARTTLS" and
                        outdata.strip().lower().startswith("ok")):
                        # STARTTLS ok, switch to secure mode
                        if test_support.verbose:
                            sys.stdout.write(
                                " client:  read %s from server, starting TLS...\n"
                                % repr(outdata))
                        conn = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
                        wrapped = True
                    elif (indata == "ENDTLS" and
                        outdata.strip().lower().startswith("ok")):
                        # ENDTLS ok, switch back to clear text
                        if test_support.verbose:
                            sys.stdout.write(
                                " client:  read %s from server, ending TLS...\n"
                                % repr(outdata))
                        s = conn.unwrap()
                        wrapped = False
                    else:
                        if test_support.verbose:
                            sys.stdout.write(
                                " client:  read %s from server\n" % repr(outdata))
                if test_support.verbose:
                    sys.stdout.write(" client:  closing connection.\n")
                if wrapped:
                    conn.write("over\n")
                else:
                    s.send("over\n")
                s.close()

        def test_socketserver(self):
            """Using a SocketServer to create and manage SSL connections."""
            server = SocketServerHTTPSServer(CERTFILE)
            flag = threading.Event()
            server.start(flag)
            # wait for it to start
            flag.wait()
            # try to connect
            try:
                if test_support.verbose:
                    sys.stdout.write('\n')
                with open(CERTFILE, 'rb') as f:
                    d1 = f.read()
                d2 = ''
                # now fetch the same data from the HTTPS server
                url = 'https://127.0.0.1:%d/%s' % (
                    server.port, os.path.split(CERTFILE)[1])
                with test_support.check_py3k_warnings():
                    import ssl
                    if hasattr(ssl, '_create_unverified_context'):
                        # Disable verification for our self-signed cert
                        # on Python >= 2.7.9 and 3.4
                        ssl_ctx = ssl._create_unverified_context()
                        f = urllib.urlopen(url, context=ssl_ctx)
                    else:
                        f = urllib.urlopen(url)
                dlen = f.info().getheader("content-length")
                if dlen and (int(dlen) > 0):
                    d2 = f.read(int(dlen))
                    if test_support.verbose:
                        sys.stdout.write(
                            " client: read %d bytes from remote server '%s'\n"
                            % (len(d2), server))
                f.close()
                self.assertEqual(d1, d2)
            finally:
                server.stop()
                server.join()

        @unittest.skipIf(hasattr(ssl, 'SSLContext'), "<= 2.7.9 only")
        def test_wrapped_accept(self):
            """Check the accept() method on SSL sockets."""
            if test_support.verbose:
                sys.stdout.write("\n")
            server_params_test(CERTFILE, ssl.PROTOCOL_SSLv23, ssl.CERT_REQUIRED,
                               CERTFILE, CERTFILE, ssl.PROTOCOL_SSLv23,
                               chatty=True, connectionchatty=True,
                               wrap_accepting_socket=True)

        def test_asyncore_server(self):
            """Check the example asyncore integration."""
            indata = "TEST MESSAGE of mixed case\n"

            if test_support.verbose:
                sys.stdout.write("\n")
            server = AsyncoreEchoServer(CERTFILE)
            with server:
                s = ssl.wrap_socket(socket.socket())
                s.connect(('127.0.0.1', server.port))
                if test_support.verbose:
                    sys.stdout.write(
                        " client:  sending %s...\n" % (repr(indata)))
                s.write(indata)
                outdata = s.read()
                if test_support.verbose:
                    sys.stdout.write(" client:  read %s\n" % repr(outdata))
                if outdata != indata.lower():
                    self.fail(
                        "bad data <<%s>> (%d) received; expected <<%s>> (%d)\n"
                        % (outdata[:min(len(outdata),20)], len(outdata),
                           indata[:min(len(indata),20)].lower(), len(indata)))
                s.write("over\n")
                if test_support.verbose:
                    sys.stdout.write(" client:  closing connection.\n")
                s.close()

        def test_recv_send(self):
            """Test recv(), send() and friends."""
            if test_support.verbose:
                sys.stdout.write("\n")

            server = ThreadedEchoServer(CERTFILE,
                                        certreqs=ssl.CERT_NONE,
                                        ssl_version=ssl.PROTOCOL_TLSv1,
                                        cacerts=CERTFILE,
                                        chatty=True,
                                        connectionchatty=False)
            with server:
                s = ssl.wrap_socket(socket.socket(),
                                    server_side=False,
                                    certfile=CERTFILE,
                                    ca_certs=CERTFILE,
                                    cert_reqs=ssl.CERT_NONE,
                                    ssl_version=ssl.PROTOCOL_TLSv1)
                s.connect((HOST, server.port))
                # helper methods for standardising recv* method signatures
                def _recv_into():
                    b = bytearray("\0"*100)
                    count = s.recv_into(b)
                    return b[:count]

                def _recvfrom_into():
                    b = bytearray("\0"*100)
                    count, addr = s.recvfrom_into(b)
                    return b[:count]

                # (name, method, whether to expect success, *args)
                send_methods = [
                    ('send', s.send, True, []),
                    ('sendto', s.sendto, False, ["some.address"]),
                    ('sendall', s.sendall, True, []),
                ]
                recv_methods = [
                    ('recv', s.recv, True, []),
                    ('recvfrom', s.recvfrom, False, ["some.address"]),
                    ('recv_into', _recv_into, True, []),
                    ('recvfrom_into', _recvfrom_into, False, []),
                ]
                data_prefix = u"PREFIX_"

                for meth_name, send_meth, expect_success, args in send_methods:
                    indata = data_prefix + meth_name
                    try:
                        send_meth(indata.encode('ASCII', 'strict'), *args)
                        outdata = s.read()
                        outdata = outdata.decode('ASCII', 'strict')
                        if outdata != indata.lower():
                            self.fail(
                                "While sending with <<%s>> bad data "
                                "<<%r>> (%d) received; "
                                "expected <<%r>> (%d)\n" % (
                                    meth_name, outdata[:20], len(outdata),
                                    indata[:20], len(indata)
                                )
                            )
                    except ValueError as e:
                        if expect_success:
                            self.fail(
                                "Failed to send with method <<%s>>; "
                                "expected to succeed.\n" % (meth_name,)
                            )
                        if not str(e).startswith(meth_name):
                            self.fail(
                                "Method <<%s>> failed with unexpected "
                                "exception message: %s\n" % (
                                    meth_name, e
                                )
                            )

                for meth_name, recv_meth, expect_success, args in recv_methods:
                    indata = data_prefix + meth_name
                    try:
                        s.send(indata.encode('ASCII', 'strict'))
                        outdata = recv_meth(*args)
                        outdata = outdata.decode('ASCII', 'strict')
                        if outdata != indata.lower():
                            self.fail(
                                "While receiving with <<%s>> bad data "
                                "<<%r>> (%d) received; "
                                "expected <<%r>> (%d)\n" % (
                                    meth_name, outdata[:20], len(outdata),
                                    indata[:20], len(indata)
                                )
                            )
                    except ValueError as e:
                        if expect_success:
                            self.fail(
                                "Failed to receive with method <<%s>>; "
                                "expected to succeed.\n" % (meth_name,)
                            )
                        if not str(e).startswith(meth_name):
                            self.fail(
                                "Method <<%s>> failed with unexpected "
                                "exception message: %s\n" % (
                                    meth_name, e
                                )
                            )
                        # consume data
                        s.read()

                s.write("over\n".encode("ASCII", "strict"))
                s.close()

        def test_handshake_timeout(self):
            # Issue #5103: SSL handshake must respect the socket timeout
            server = socket.socket(socket.AF_INET)
            host = "127.0.0.1"
            port = test_support.bind_port(server)
            started = threading.Event()
            finish = False

            def serve():
                server.listen(5)
                started.set()
                conns = []
                while not finish:
                    r, w, e = select.select([server], [], [], 0.1)
                    if server in r:
                        # Let the socket hang around rather than having
                        # it closed by garbage collection.
                        conns.append(server.accept()[0])

            t = threading.Thread(target=serve)
            t.start()
            started.wait()

            try:
                try:
                    c = socket.socket(socket.AF_INET)
                    c.settimeout(0.2)
                    c.connect((host, port))
                    # Will attempt handshake and time out
                    self.assertRaisesRegexp(ssl.SSLError, "timed out",
                                            ssl.wrap_socket, c)
                finally:
                    c.close()
                try:
                    c = socket.socket(socket.AF_INET)
                    c.settimeout(0.2)
                    c = ssl.wrap_socket(c)
                    # Will attempt handshake and time out
                    self.assertRaisesRegexp(ssl.SSLError, "timed out",
                                            c.connect, (host, port))
                finally:
                    c.close()
            finally:
                finish = True
                t.join()
                server.close()

        def test_default_ciphers(self):
            with ThreadedEchoServer(CERTFILE,
                                    ssl_version=ssl.PROTOCOL_SSLv23,
                                    chatty=False) as server:
                sock = socket.socket()
                try:
                    # Force a set of weak ciphers on our client socket
                    try:
                        s = ssl.wrap_socket(sock,
                                            ssl_version=ssl.PROTOCOL_SSLv23,
                                            ciphers="DES")
                    except ssl.SSLError:
                        self.skipTest("no DES cipher available")
                    with self.assertRaises((OSError, ssl.SSLError)):
                        s.connect((HOST, server.port))
                finally:
                    sock.close()
            self.assertIn("no shared cipher", str(server.conn_errors[0]))


def test_main(verbose=False):
    global CERTFILE, SVN_PYTHON_ORG_ROOT_CERT, NOKIACERT
    CERTFILE = os.path.join(os.path.dirname(__file__) or os.curdir,
                            "keycert.pem")
    SVN_PYTHON_ORG_ROOT_CERT = os.path.join(
        os.path.dirname(__file__) or os.curdir,
        "https_svn_python_org_root.pem")
    NOKIACERT = os.path.join(os.path.dirname(__file__) or os.curdir,
                             "nokia.pem")

    if (not os.path.exists(CERTFILE) or
        not os.path.exists(SVN_PYTHON_ORG_ROOT_CERT) or
        not os.path.exists(NOKIACERT)):
        raise test_support.TestFailed("Can't read certificate files!")

    tests = [BasicTests, BasicSocketTests]

    if test_support.is_resource_enabled('network'):
        tests.append(NetworkedTests)

    if _have_threads:
        thread_info = test_support.threading_setup()
        if thread_info and test_support.is_resource_enabled('network'):
            tests.append(ThreadedTests)

    try:
        test_support.run_unittest(*tests)
    finally:
        if _have_threads:
            test_support.threading_cleanup(*thread_info)

if __name__ == "__main__":
    test_main()



import os
from gevent.pywsgi import WSGIServer
from gevent import socket


def application(environ, start_response):
    start_response('200 OK', [])
    return []


if __name__ == '__main__':
    listener = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sockname = './' + os.path.basename(__file__) + '.sock'
    if os.path.exists(sockname):
        os.remove(sockname)
    listener.bind(sockname)
    listener.listen(1)
    WSGIServer(listener, application).serve_forever()



# Generated by cythonpp.py on 2015-09-18 08:28:24
# Copyright (c) 2009-2012 Denis Bilenko. See LICENSE for details.
cimport cython
cimport libev
from python cimport *
# Work around lack of absolute_import in Cython
# Note for PY3: not doing so will leave reference to locals() on import
# (reproducible under Python 3.3, not under Python 3.4; see test__refcount_core.py)
sys = __import__('sys', level=0)
os = __import__('os', level=0)
traceback = __import__('traceback', level=0)
signalmodule = __import__('signal', level=0)


__all__ = ['get_version',
           'get_header_version',
           'supported_backends',
           'recommended_backends',
           'embeddable_backends',
           'time',
           'loop']

cdef tuple integer_types

if sys.version_info[0] >= 3:
    integer_types = int,
else:
    integer_types = (int, long)


cdef extern from "callbacks.h":
    void gevent_callback_io(libev.ev_loop, void*, int)
    void gevent_callback_timer(libev.ev_loop, void*, int)
    void gevent_callback_signal(libev.ev_loop, void*, int)
    void gevent_callback_idle(libev.ev_loop, void*, int)
    void gevent_callback_prepare(libev.ev_loop, void*, int)
    void gevent_callback_check(libev.ev_loop, void*, int)
    void gevent_callback_fork(libev.ev_loop, void*, int)
    void gevent_callback_async(libev.ev_loop, void*, int)
    void gevent_callback_child(libev.ev_loop, void*, int)
    void gevent_callback_stat(libev.ev_loop, void*, int)
    void gevent_run_callbacks(libev.ev_loop, void*, int)
    void gevent_periodic_signal_check(libev.ev_loop, void*, int)
    void gevent_call(loop, callback)
    void gevent_noop(libev.ev_loop, void*, int)

cdef extern from *:
    int errno

cdef extern from "stathelper.c":
    object _pystat_fromstructstat(void*)


UNDEF = libev.EV_UNDEF
NONE = libev.EV_NONE
READ = libev.EV_READ
WRITE = libev.EV_WRITE
TIMER = libev.EV_TIMER
PERIODIC = libev.EV_PERIODIC
SIGNAL = libev.EV_SIGNAL
CHILD = libev.EV_CHILD
STAT = libev.EV_STAT
IDLE = libev.EV_IDLE
PREPARE = libev.EV_PREPARE
CHECK = libev.EV_CHECK
EMBED = libev.EV_EMBED
FORK = libev.EV_FORK
CLEANUP = libev.EV_CLEANUP
ASYNC = libev.EV_ASYNC
CUSTOM = libev.EV_CUSTOM
ERROR = libev.EV_ERROR

READWRITE = libev.EV_READ | libev.EV_WRITE

MINPRI = libev.EV_MINPRI
MAXPRI = libev.EV_MAXPRI

BACKEND_PORT = libev.EVBACKEND_PORT
BACKEND_KQUEUE = libev.EVBACKEND_KQUEUE
BACKEND_EPOLL = libev.EVBACKEND_EPOLL
BACKEND_POLL = libev.EVBACKEND_POLL
BACKEND_SELECT = libev.EVBACKEND_SELECT
FORKCHECK = libev.EVFLAG_FORKCHECK
NOINOTIFY = libev.EVFLAG_NOINOTIFY
SIGNALFD = libev.EVFLAG_SIGNALFD
NOSIGMASK = libev.EVFLAG_NOSIGMASK


@cython.internal
cdef class _EVENTSType:

    def __repr__(self):
        return 'gevent.core.EVENTS'


cdef public object GEVENT_CORE_EVENTS = _EVENTSType()
EVENTS = GEVENT_CORE_EVENTS


def get_version():
    return 'libev-%d.%02d' % (libev.ev_version_major(), libev.ev_version_minor())


def get_header_version():
    return 'libev-%d.%02d' % (libev.EV_VERSION_MAJOR, libev.EV_VERSION_MINOR)


# This list backends in the order they are actually tried by libev
_flags = [(libev.EVBACKEND_PORT, 'port'),
          (libev.EVBACKEND_KQUEUE, 'kqueue'),
          (libev.EVBACKEND_EPOLL, 'epoll'),
          (libev.EVBACKEND_POLL, 'poll'),
          (libev.EVBACKEND_SELECT, 'select'),
          (libev.EVFLAG_NOENV, 'noenv'),
          (libev.EVFLAG_FORKCHECK, 'forkcheck'),
          (libev.EVFLAG_NOINOTIFY, 'noinotify'),
          (libev.EVFLAG_SIGNALFD, 'signalfd'),
          (libev.EVFLAG_NOSIGMASK, 'nosigmask')]


_flags_str2int = dict((string, flag) for (flag, string) in _flags)


_events = [(libev.EV_READ,     'READ'),
           (libev.EV_WRITE,    'WRITE'),
           (libev.EV__IOFDSET, '_IOFDSET'),
           (libev.EV_PERIODIC, 'PERIODIC'),
           (libev.EV_SIGNAL,   'SIGNAL'),
           (libev.EV_CHILD,    'CHILD'),
           (libev.EV_STAT,     'STAT'),
           (libev.EV_IDLE,     'IDLE'),
           (libev.EV_PREPARE,  'PREPARE'),
           (libev.EV_CHECK,    'CHECK'),
           (libev.EV_EMBED,    'EMBED'),
           (libev.EV_FORK,     'FORK'),
           (libev.EV_CLEANUP,  'CLEANUP'),
           (libev.EV_ASYNC,    'ASYNC'),
           (libev.EV_CUSTOM,   'CUSTOM'),
           (libev.EV_ERROR,    'ERROR')]


cpdef _flags_to_list(unsigned int flags):
    cdef list result = []
    for code, value in _flags:
        if flags & code:
            result.append(value)
        flags &= ~code
        if not flags:
            break
    if flags:
        result.append(flags)
    return result


if sys.version_info[0] >= 3:
    basestring = (bytes, str)
else:
    basestring = __builtins__.basestring


cpdef unsigned int _flags_to_int(object flags) except? -1:
    # Note, that order does not matter, libev has its own predefined order
    if not flags:
        return 0
    if isinstance(flags, integer_types):
        return flags
    cdef unsigned int result = 0
    try:
        if isinstance(flags, basestring):
            flags = flags.split(',')
        for value in flags:
            value = value.strip().lower()
            if value:
                result |= _flags_str2int[value]
    except KeyError as ex:
        raise ValueError('Invalid backend or flag: %s\nPossible values: %s' % (ex, ', '.join(sorted(_flags_str2int.keys()))))
    return result


cdef str _str_hex(object flag):
    if isinstance(flag, integer_types):
        return hex(flag)
    return str(flag)


cpdef _check_flags(unsigned int flags):
    cdef list as_list
    flags &= libev.EVBACKEND_MASK
    if not flags:
        return
    if not (flags & libev.EVBACKEND_ALL):
        raise ValueError('Invalid value for backend: 0x%x' % flags)
    if not (flags & libev.ev_supported_backends()):
        as_list = [_str_hex(x) for x in _flags_to_list(flags)]
        raise ValueError('Unsupported backend: %s' % '|'.join(as_list))


cpdef _events_to_str(int events):
    cdef list result = []
    cdef int c_flag
    for (flag, string) in _events:
        c_flag = flag
        if events & c_flag:
            result.append(string)
            events = events & (~c_flag)
        if not events:
            break
    if events:
        result.append(hex(events))
    return '|'.join(result)


def supported_backends():
    return _flags_to_list(libev.ev_supported_backends())


def recommended_backends():
    return _flags_to_list(libev.ev_recommended_backends())


def embeddable_backends():
    return _flags_to_list(libev.ev_embeddable_backends())


def time():
    return libev.ev_time()




cdef bint _default_loop_destroyed = False







cdef public class loop [object PyGeventLoopObject, type PyGeventLoop_Type]:
    cdef libev.ev_loop* _ptr
    cdef public object error_handler
    cdef libev.ev_prepare _prepare
    cdef public list _callbacks
    cdef libev.ev_timer _timer0
#ifdef _WIN32
    cdef libev.ev_timer _periodic_signal_checker
#endif

    def __init__(self, object flags=None, object default=None, size_t ptr=0):
        cdef unsigned int c_flags
        cdef object old_handler = None
        libev.ev_prepare_init(&self._prepare, <void*>gevent_run_callbacks)
#ifdef _WIN32
        libev.ev_timer_init(&self._periodic_signal_checker, <void*>gevent_periodic_signal_check, 0.3, 0.3)
#endif
        libev.ev_timer_init(&self._timer0, <void*>gevent_noop, 0.0, 0.0)
        if ptr:
            self._ptr = <libev.ev_loop*>ptr
        else:
            c_flags = _flags_to_int(flags)
            _check_flags(c_flags)
            c_flags |= libev.EVFLAG_NOENV
            c_flags |= libev.EVFLAG_FORKCHECK
            if default is None:
                default = True
                if _default_loop_destroyed:
                    default = False
            if default:
                self._ptr = libev.gevent_ev_default_loop(c_flags)
                if not self._ptr:
                    raise SystemError("ev_default_loop(%s) failed" % (c_flags, ))
#ifdef _WIN32
                libev.ev_timer_start(self._ptr, &self._periodic_signal_checker)
                libev.ev_unref(self._ptr)
#endif
            else:
                self._ptr = libev.ev_loop_new(c_flags)
                if not self._ptr:
                    raise SystemError("ev_loop_new(%s) failed" % (c_flags, ))
            if default or __SYSERR_CALLBACK is None:
                set_syserr_cb(self._handle_syserr)
            libev.ev_prepare_start(self._ptr, &self._prepare)
            libev.ev_unref(self._ptr)
        self._callbacks = []

    cdef _run_callbacks(self):
        cdef callback cb
        cdef object callbacks
        cdef int count = 1000
        libev.ev_timer_stop(self._ptr, &self._timer0)
        while self._callbacks and count > 0:
            callbacks = self._callbacks
            self._callbacks = []
            for cb in callbacks:
                libev.ev_unref(self._ptr)
                gevent_call(self, cb)
                count -= 1
        if self._callbacks:
            libev.ev_timer_start(self._ptr, &self._timer0)

    def _stop_watchers(self):
        if libev.ev_is_active(&self._prepare):
            libev.ev_ref(self._ptr)
            libev.ev_prepare_stop(self._ptr, &self._prepare)
#ifdef _WIN32
        if libev.ev_is_active(&self._periodic_signal_checker):
            libev.ev_ref(self._ptr)
            libev.ev_timer_stop(self._ptr, &self._periodic_signal_checker)
#endif

    def destroy(self):
        global _default_loop_destroyed
        if self._ptr:
            self._stop_watchers()
            if __SYSERR_CALLBACK == self._handle_syserr:
                set_syserr_cb(None)
            if libev.ev_is_default_loop(self._ptr):
                _default_loop_destroyed = True
            libev.ev_loop_destroy(self._ptr)
            self._ptr = NULL

    def __dealloc__(self):
        if self._ptr:
            self._stop_watchers()
            if not libev.ev_is_default_loop(self._ptr):
                libev.ev_loop_destroy(self._ptr)
            self._ptr = NULL

    property ptr:

        def __get__(self):
            return <size_t>self._ptr

    property WatcherType:

        def __get__(self):
            return watcher

    property MAXPRI:

       def __get__(self):
           return libev.EV_MAXPRI

    property MINPRI:

        def __get__(self):
            return libev.EV_MINPRI

    def _handle_syserr(self, message, errno):
        if sys.version_info[0] >= 3:
            message = message.decode()
        self.handle_error(None, SystemError, SystemError(message + ': ' + os.strerror(errno)), None)

    cpdef handle_error(self, context, type, value, tb):
        cdef object handle_error
        cdef object error_handler = self.error_handler
        if error_handler is not None:
            # we do want to do getattr every time so that setting Hub.handle_error property just works
            handle_error = getattr(error_handler, 'handle_error', error_handler)
            handle_error(context, type, value, tb)
        else:
            self._default_handle_error(context, type, value, tb)

    cpdef _default_handle_error(self, context, type, value, tb):
        # note: Hub sets its own error handler so this is not used by gevent
        # this is here to make core.loop usable without the rest of gevent
        traceback.print_exception(type, value, tb)
        if self._ptr:
            libev.ev_break(self._ptr, libev.EVBREAK_ONE)

    def run(self, nowait=False, once=False):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        cdef unsigned int flags = 0
        if nowait:
            flags |= libev.EVRUN_NOWAIT
        if once:
            flags |= libev.EVRUN_ONCE
        with nogil:
            libev.ev_run(self._ptr, flags)

    def reinit(self):
        if self._ptr:
            libev.ev_loop_fork(self._ptr)

    def ref(self):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_ref(self._ptr)

    def unref(self):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_unref(self._ptr)

    def break_(self, int how=libev.EVBREAK_ONE):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_break(self._ptr, how)

    def verify(self):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_verify(self._ptr)

    def now(self):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        return libev.ev_now(self._ptr)

    def update(self):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_now_update(self._ptr)

    def __repr__(self):
        return '<%s at 0x%x %s>' % (self.__class__.__name__, id(self), self._format())

    property default:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return True if libev.ev_is_default_loop(self._ptr) else False

    property iteration:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return libev.ev_iteration(self._ptr)

    property depth:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return libev.ev_depth(self._ptr)

    property backend_int:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return libev.ev_backend(self._ptr)

    property backend:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            cdef unsigned int backend = libev.ev_backend(self._ptr)
            for key, value in _flags:
                if key == backend:
                    return value
            return backend

    property pendingcnt:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return libev.ev_pending_count(self._ptr)

#ifdef _WIN32
    def io(self, libev.vfd_socket_t fd, int events, ref=True, priority=None):
        return io(self, fd, events, ref, priority)
#else
    def io(self, int fd, int events, ref=True, priority=None):
        return io(self, fd, events, ref, priority)
#endif

    def timer(self, double after, double repeat=0.0, ref=True, priority=None):
        return timer(self, after, repeat, ref, priority)

    def signal(self, int signum, ref=True, priority=None):
        return signal(self, signum, ref, priority)

    def idle(self, ref=True, priority=None):
        return idle(self, ref, priority)

    def prepare(self, ref=True, priority=None):
        return prepare(self, ref, priority)

    def check(self, ref=True, priority=None):
        return check(self, ref, priority)

    def fork(self, ref=True, priority=None):
        return fork(self, ref, priority)

    def async(self, ref=True, priority=None):
        return async(self, ref, priority)

#ifdef _WIN32
#else

    def child(self, int pid, bint trace=0, ref=True):
        return child(self, pid, trace, ref)

    def install_sigchld(self):
        libev.gevent_install_sigchld_handler()

#endif

    def stat(self, str path, float interval=0.0, ref=True, priority=None):
        return stat(self, path, interval, ref, priority)

    def run_callback(self, func, *args):
        
        if not self._ptr:
            raise ValueError('operation on destroyed loop')
        cdef callback cb = callback(func, args)
        self._callbacks.append(cb)
        libev.ev_ref(self._ptr)
        return cb

    def _format(self):
        if not self._ptr:
            return 'destroyed'
        cdef object msg = self.backend
        if self.default:
            msg += ' default'
        msg += ' pending=%s' % self.pendingcnt
#ifdef LIBEV_EMBED
        msg += self._format_details()
#endif
        return msg

#ifdef LIBEV_EMBED

    def _format_details(self):
        cdef str msg = ''
        cdef object fileno = self.fileno()
        cdef object sigfd = None
        cdef object activecnt = None
        try:
            sigfd = self.sigfd
        except AttributeError:
            sigfd = None
        try:
            activecnt = self.activecnt
        except AttributeError:
            pass
        if activecnt is not None:
            msg += ' ref=' + repr(activecnt)
        if fileno is not None:
            msg += ' fileno=' + repr(fileno)
        if sigfd is not None and sigfd != -1:
            msg += ' sigfd=' + repr(sigfd)
        return msg

    def fileno(self):
        cdef int fd
        if self._ptr:
            fd = self._ptr.backend_fd
            if fd >= 0:
                return fd

    property activecnt:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return self._ptr.activecnt

    property sig_pending:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return self._ptr.sig_pending

#if EV_USE_SIGNALFD
    property sigfd:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return self._ptr.sigfd
#endif

    property origflags:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return _flags_to_list(self._ptr.origflags)

    property origflags_int:

        def __get__(self):
            
            if not self._ptr:
                raise ValueError('operation on destroyed loop')
            return self._ptr.origflags

#endif


cdef public class callback [object PyGeventCallbackObject, type PyGeventCallback_Type]:
    cdef public object callback
    cdef public tuple args

    def __init__(self, callback, args):
        self.callback = callback
        self.args = args

    def stop(self):
        self.callback = None
        self.args = None

    # Note, that __nonzero__ and pending are different
    # nonzero is used in contexts where we need to know whether to schedule another callback,
    # so it's true if it's pending or currently running
    # 'pending' has the same meaning as libev watchers: it is cleared before entering callback

    def __nonzero__(self):
        # it's nonzero if it's pending or currently executing
        return self.args is not None

    property pending:

        def __get__(self):
            return self.callback is not None

    def __repr__(self):
        if Py_ReprEnter(<PyObjectPtr>self) != 0:
            return "<...>"
        try:
            format = self._format()
            result = "<%s at 0x%x%s" % (self.__class__.__name__, id(self), format)
            if self.pending:
                result += " pending"
            if self.callback is not None:
                result += " callback=%r" % (self.callback, )
            if self.args is not None:
                result += " args=%r" % (self.args, )
            if self.callback is None and self.args is None:
                result += " stopped"
            return result + ">"
        finally:
            Py_ReprLeave(<PyObjectPtr>self)

    def _format(self):
        return ''




# about readonly _flags attribute:
# bit #1 set if object owns Python reference to itself (Py_INCREF was called and we must call Py_DECREF later)
# bit #2 set if ev_unref() was called and we must call ev_ref() later
# bit #3 set if user wants to call ev_unref() before start()














cdef public class watcher [object PyGeventWatcherObject, type PyGeventWatcher_Type]:
    """Abstract base class for all the watchers"""

    def __repr__(self):
        if Py_ReprEnter(<PyObjectPtr>self) != 0:
            return "<...>"
        try:
            format = self._format()
            result = "<%s at 0x%x%s" % (self.__class__.__name__, id(self), format)
            if self.active:
                result += " active"
            if self.pending:
                result += " pending"
            if self.callback is not None:
                result += " callback=%r" % (self.callback, )
            if self.args is not None:
                result += " args=%r" % (self.args, )
            return result + ">"
        finally:
            Py_ReprLeave(<PyObjectPtr>self)

    def _format(self):
        return ''


cdef public class io(watcher) [object PyGeventIOObject, type PyGeventIO_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_io _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_io_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args, pass_events=False):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        if pass_events:
            self.args = (GEVENT_CORE_EVENTS, ) + args
        else:
            self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_io_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

#ifdef _WIN32

    def __init__(self, loop loop, libev.vfd_socket_t fd, int events, ref=True, priority=None):
        if events & ~(libev.EV__IOFDSET | libev.EV_READ | libev.EV_WRITE):
            raise ValueError('illegal event mask: %r' % events)
        cdef int vfd = libev.vfd_open(fd)
        libev.vfd_free(self._watcher.fd)
        libev.ev_io_init(&self._watcher, <void *>gevent_callback_io, vfd, events)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)

#else

    def __init__(self, loop loop, int fd, int events, ref=True, priority=None):
        if fd < 0:
            raise ValueError('fd must be non-negative: %r' % fd)
        if events & ~(libev.EV__IOFDSET | libev.EV_READ | libev.EV_WRITE):
            raise ValueError('illegal event mask: %r' % events)
        libev.ev_io_init(&self._watcher, <void *>gevent_callback_io, fd, events)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)

#endif

    property fd:

        def __get__(self):
            return libev.vfd_get(self._watcher.fd)

        def __set__(self, long fd):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("'io' watcher attribute 'fd' is read-only while watcher is active")
            cdef int vfd = libev.vfd_open(fd)
            libev.vfd_free(self._watcher.fd)
            libev.ev_io_init(&self._watcher, <void *>gevent_callback_io, vfd, self._watcher.events)

    property events:

        def __get__(self):
            return self._watcher.events

        def __set__(self, int events):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("'io' watcher attribute 'events' is read-only while watcher is active")
            libev.ev_io_init(&self._watcher, <void *>gevent_callback_io, self._watcher.fd, events)

    property events_str:

        def __get__(self):
            return _events_to_str(self._watcher.events)

    def _format(self):
        return ' fd=%s events=%s' % (self.fd, self.events_str)

#ifdef _WIN32

    def __cinit__(self):
        self._watcher.fd = -1;

    def __dealloc__(self):
        libev.vfd_free(self._watcher.fd)

#endif


cdef public class timer(watcher) [object PyGeventTimerObject, type PyGeventTimer_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_timer _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_timer_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args, update=True):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        if update:
            libev.ev_now_update(self.loop._ptr)
        libev.ev_timer_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    def __init__(self, loop loop, double after=0.0, double repeat=0.0, ref=True, priority=None):
        if repeat < 0.0:
            raise ValueError("repeat must be positive or zero: %r" % repeat)
        libev.ev_timer_init(&self._watcher, <void *>gevent_callback_timer, after, repeat)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)

    property at:

        def __get__(self):
            return self._watcher.at

    # QQQ: add 'after' and 'repeat' properties?

    def again(self, object callback, *args, update=True):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        if update:
            libev.ev_now_update(self.loop._ptr)
        libev.ev_timer_again(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1


cdef public class signal(watcher) [object PyGeventSignalObject, type PyGeventSignal_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_signal _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_signal_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_signal_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    def __init__(self, loop loop, int signalnum, ref=True, priority=None):
        if signalnum < 1 or signalnum >= signalmodule.NSIG:
            raise ValueError('illegal signal number: %r' % signalnum)
        # still possible to crash on one of libev's asserts:
        # 1) "libev: ev_signal_start called with illegal signal number"
        #    EV_NSIG might be different from signal.NSIG on some platforms
        # 2) "libev: a signal must not be attached to two different loops"
        #    we probably could check that in LIBEV_EMBED mode, but not in general
        libev.ev_signal_init(&self._watcher, <void *>gevent_callback_signal, signalnum)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)


cdef public class idle(watcher) [object PyGeventIdleObject, type PyGeventIdle_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_idle _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_idle_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_idle_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    
    def __init__(self, loop loop , ref=True, priority=None):
        libev.ev_idle_init(&self._watcher, <void *>gevent_callback_idle )
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)


cdef public class prepare(watcher) [object PyGeventPrepareObject, type PyGeventPrepare_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_prepare _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_prepare_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_prepare_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    
    def __init__(self, loop loop , ref=True, priority=None):
        libev.ev_prepare_init(&self._watcher, <void *>gevent_callback_prepare )
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)


cdef public class check(watcher) [object PyGeventCheckObject, type PyGeventCheck_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_check _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_check_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_check_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    
    def __init__(self, loop loop , ref=True, priority=None):
        libev.ev_check_init(&self._watcher, <void *>gevent_callback_check )
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)


cdef public class fork(watcher) [object PyGeventForkObject, type PyGeventFork_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_fork _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_fork_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_fork_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    
    def __init__(self, loop loop , ref=True, priority=None):
        libev.ev_fork_init(&self._watcher, <void *>gevent_callback_fork )
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)


cdef public class async(watcher) [object PyGeventAsyncObject, type PyGeventAsync_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_async _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_async_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_async_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    property pending:

        def __get__(self):
            return True if libev.ev_async_pending(&self._watcher) else False

    
    def __init__(self, loop loop , ref=True, priority=None):
        libev.ev_async_init(&self._watcher, <void *>gevent_callback_async )
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)

    def send(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        libev.ev_async_send(self.loop._ptr, &self._watcher)

#ifdef _WIN32
#else

cdef public class child(watcher) [object PyGeventChildObject, type PyGeventChild_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_child _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_child_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_child_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False

    def __init__(self, loop loop, int pid, bint trace=0, ref=True):
        if not loop.default:
            raise TypeError('child watchers are only available on the default loop')
        libev.gevent_install_sigchld_handler()
        libev.ev_child_init(&self._watcher, <void *>gevent_callback_child, pid, trace)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4

    def _format(self):
        return ' pid=%r rstatus=%r' % (self.pid, self.rstatus)

    property pid:

        def __get__(self):
            return self._watcher.pid

    property rpid:

        def __get__(self):
            return self._watcher.rpid

        def __set__(self, int value):
            self._watcher.rpid = value

    property rstatus:

        def __get__(self):
            return self._watcher.rstatus

        def __set__(self, int value):
            self._watcher.rstatus = value

#endif


cdef public class stat(watcher) [object PyGeventStatObject, type PyGeventStat_Type]:

    
    cdef public loop loop
    cdef object _callback
    cdef public tuple args
    cdef readonly int _flags
    cdef libev.ev_stat _watcher

    property ref:

        def __get__(self):
            return False if self._flags & 4 else True

        def __set__(self, object value):
            
            if not self.loop._ptr:
                raise ValueError('operation on destroyed loop')
            if value:
                if not self._flags & 4:
                    return  # ref is already True
                if self._flags & 2:  # ev_unref was called, undo
                    libev.ev_ref(self.loop._ptr)
                self._flags &= ~6  # do not want unref, no outstanding unref
            else:
                if self._flags & 4:
                    return  # ref is already False
                self._flags |= 4
                if not self._flags & 2 and libev.ev_is_active(&self._watcher):
                    libev.ev_unref(self.loop._ptr)
                    self._flags |= 2

    property callback:

        def __get__(self):
            return self._callback

        def __set__(self, object callback):
            if not PyCallable_Check(<PyObjectPtr>callback) and callback is not None:
                raise TypeError("Expected callable, not %r" % (callback, ))
            self._callback = callback

    def stop(self):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if self._flags & 2:
            libev.ev_ref(self.loop._ptr)
            self._flags &= ~2
        libev.ev_stat_stop(self.loop._ptr, &self._watcher)
        self._callback = None
        self.args = None
        if self._flags & 1:
            Py_DECREF(<PyObjectPtr>self)
            self._flags &= ~1

    property priority:

        def __get__(self):
            return libev.ev_priority(&self._watcher)

        def __set__(self, int priority):
            if libev.ev_is_active(&self._watcher):
                raise AttributeError("Cannot set priority of an active watcher")
            libev.ev_set_priority(&self._watcher, priority)

    def feed(self, int revents, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_feed_event(self.loop._ptr, &self._watcher, revents)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    def start(self, object callback, *args):
        
        if not self.loop._ptr:
            raise ValueError('operation on destroyed loop')
        if callback is None:
            raise TypeError('callback must be callable, not None')
        self.callback = callback
        self.args = args
        if self._flags & 6 == 4:
            libev.ev_unref(self.loop._ptr)
            self._flags |= 2
        libev.ev_stat_start(self.loop._ptr, &self._watcher)
        if not self._flags & 1:
            Py_INCREF(<PyObjectPtr>self)
            self._flags |= 1

    property active:

        def __get__(self):
            return True if libev.ev_is_active(&self._watcher) else False

    
    property pending:

        def __get__(self):
            return True if libev.ev_is_pending(&self._watcher) else False
    cdef readonly str path
    cdef readonly bytes _paths

    def __init__(self, loop loop, str path, float interval=0.0, ref=True, priority=None):
        self.path = path
        cdef bytes paths
        if isinstance(path, unicode):
            # the famous Python3 filesystem encoding debacle hits us here. Can we do better?
            # We must keep a reference to the encoded string so that its bytes don't get freed
            # and overwritten, leading to strange errors from libev ("no such file or directory")
            paths = (<unicode>path).encode(sys.getfilesystemencoding())
            self._paths = paths
        else:
            paths = <bytes>path
        libev.ev_stat_init(&self._watcher, <void *>gevent_callback_stat, <char*>paths, interval)
        self.loop = loop
        if ref:
            self._flags = 0
        else:
            self._flags = 4
        if priority is not None:
            libev.ev_set_priority(&self._watcher, priority)

    property attr:

        def __get__(self):
            if not self._watcher.attr.st_nlink:
                return
            return _pystat_fromstructstat(&self._watcher.attr)

    property prev:

       def __get__(self):
            if not self._watcher.prev.st_nlink:
                return
            return _pystat_fromstructstat(&self._watcher.prev)

    property interval:

        def __get__(self):
            return self._watcher.interval


__SYSERR_CALLBACK = None


cdef void _syserr_cb(char* msg) with gil:
    try:
        __SYSERR_CALLBACK(msg, errno)
    except:
        set_syserr_cb(None)
        print_exc = getattr(traceback, 'print_exc', None)
        if print_exc is not None:
            print_exc()


cpdef set_syserr_cb(callback):
    global __SYSERR_CALLBACK
    if callback is None:
        libev.ev_set_syserr_cb(NULL)
        __SYSERR_CALLBACK = None
    elif callable(callback):
        libev.ev_set_syserr_cb(<void *>_syserr_cb)
        __SYSERR_CALLBACK = callback
    else:
        raise TypeError('Expected callable or None, got %r' % (callback, ))


#ifdef LIBEV_EMBED
LIBEV_EMBED = True
EV_USE_FLOOR = libev.EV_USE_FLOOR
EV_USE_CLOCK_SYSCALL = libev.EV_USE_CLOCK_SYSCALL
EV_USE_REALTIME = libev.EV_USE_REALTIME
EV_USE_MONOTONIC = libev.EV_USE_MONOTONIC
EV_USE_NANOSLEEP = libev.EV_USE_NANOSLEEP
EV_USE_INOTIFY = libev.EV_USE_INOTIFY
EV_USE_SIGNALFD = libev.EV_USE_SIGNALFD
EV_USE_EVENTFD = libev.EV_USE_EVENTFD
EV_USE_4HEAP = libev.EV_USE_4HEAP
#else
LIBEV_EMBED = False
#endif



import signal
import greentest
import gevent


class Expected(Exception):
    pass


def raise_Expected():
    raise Expected('TestSignal')


if hasattr(signal, 'SIGALRM'):

    class TestSignal(greentest.TestCase):

        error_fatal = False
        __timeout__ = 5

        def test(self):
            sig = gevent.signal(signal.SIGALRM, raise_Expected)
            assert sig.ref is False, repr(sig.ref)
            sig.ref = True
            assert sig.ref is True
            sig.ref = False
            try:
                signal.alarm(1)
                try:
                    gevent.sleep(2)
                    raise AssertionError('must raise Expected')
                except Expected as ex:
                    assert str(ex) == 'TestSignal', ex
                # also let's check that alarm is persistent
                signal.alarm(1)
                try:
                    gevent.sleep(2)
                    raise AssertionError('must raise Expected')
                except Expected as ex:
                    assert str(ex) == 'TestSignal', ex
            finally:
                sig.cancel()


if __name__ == '__main__':
    greentest.main()



import sys
#import time
from unittest import main
if sys.version_info[0] == 3:
    from urllib import request as urllib2
else:
    import urllib2

import util
import socket
import ssl


class Test_wsgiserver(util.TestServer):
    server = 'wsgiserver.py'
    URL = 'http://127.0.0.1:8088'
    PORT = 8088
    not_found_message = b'<h1>Not Found</h1>'
    ssl_ctx = None
    _use_ssl = False

    def read(self, path='/'):
        url = self.URL + path
        try:
            if self.ssl_ctx is not None:
                response = urllib2.urlopen(url, None, 2, context=self.ssl_ctx)
            else:
                response = urllib2.urlopen(url, None, 2)
        except urllib2.HTTPError:
            response = sys.exc_info()[1]
        result = '%s %s' % (response.code, response.msg), response.read()
        # XXX: It looks like under PyPy this isn't directly closing the socket
        # when SSL is in use. It takes a GC cycle to make that true.
        response.close()
        return result

    def _test_hello(self):
        status, data = self.read('/')
        self.assertEqual(status, '200 OK')
        self.assertEqual(data, b"<b>hello world</b>")

    def _test_not_found(self):
        status, data = self.read('/xxx')
        self.assertEqual(status, '404 Not Found')
        self.assertEqual(data, self.not_found_message)

    def _do_test_a_blocking_client(self):
        # We spawn this in a separate server because if it's broken
        # the whole server hangs
        print("Checking blocking")
        with self.running_server():
            # First, make sure we can talk to it.
            self._test_hello()
            # Now create a connection and only partway finish
            # the transaction
            sock = socket.create_connection(('127.0.0.1', self.PORT))
            ssl_sock = None
            if self._use_ssl:
                ssl_sock = ssl.wrap_socket(sock)
                sock_file = ssl_sock.makefile(mode='rwb')
            else:
                sock_file = sock.makefile(mode='rwb')
            # write an incomplete request
            sock_file.write(b'GET /xxx HTTP/1.0\r\n')
            sock_file.flush()
            # Leave it open and not doing anything
            # while the other request runs to completion.
            # This demonstrates that a blocking client
            # doesn't hang the whole server
            self._test_hello()

            # now finish the original request
            sock_file.write(b'\r\n')
            sock_file.flush()
            line = sock_file.readline()
            self.assertEqual(line, b'HTTP/1.1 404 Not Found\r\n')

            sock.close()
            sock_file.close()
            if ssl_sock is not None:
                ssl_sock.close()

    def test_a_blocking_client(self):
        self._do_test_a_blocking_client()


class Test_wsgiserver_ssl(Test_wsgiserver):
    server = 'wsgiserver_ssl.py'
    URL = 'https://127.0.0.1:8443'
    PORT = 8443
    _use_ssl = True

    if hasattr(ssl, '_create_unverified_context'):
        # Disable verification for our self-signed cert
        # on Python >= 2.7.9 and 3.4
        ssl_ctx = ssl._create_unverified_context()


class Test_webproxy(Test_wsgiserver):
    server = 'webproxy.py'

    def _run_all_tests(self):
        status, data = self.read('/')
        self.assertEqual(status, '200 OK')
        assert b"gevent example" in data, repr(data)
        status, data = self.read('/http://www.google.com')
        self.assertEqual(status, '200 OK')
        assert b'google' in data.lower(), repr(data)

    def _do_test_a_blocking_client(self):
        # Not applicable
        return


# class Test_webpy(Test_wsgiserver):
#     server = 'webpy.py'
#     not_found_message = 'not found'
#
#     def _test_hello(self):
#         status, data = self.read('/')
#         self.assertEqual(status, '200 OK')
#         assert "Hello, world" in data, repr(data)
#
#     def _test_long(self):
#         start = time.time()
#         status, data = self.read('/long')
#         delay = time.time() - start
#         assert 10 - 0.5 < delay < 10 + 0.5, delay
#         self.assertEqual(status, '200 OK')
#         self.assertEqual(data, 'Hello, 10 seconds later')


if __name__ == '__main__':
    main()



# mostly tests from test_subprocess.py that used to have problems
import sys
import os
import errno
import greentest
import gevent
from gevent import subprocess
import time
import gc


PYPY = hasattr(sys, 'pypy_version_info')
PY3 = sys.version_info[0] >= 3


if subprocess.mswindows:
    SETBINARY = 'import msvcrt; msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY);'
else:
    SETBINARY = ''


python_universal_newlines = hasattr(sys.stdout, 'newlines')


class Test(greentest.TestCase):

    def setUp(self):
        gc.collect()
        gc.collect()

    def test_exit(self):
        popen = subprocess.Popen([sys.executable, '-c', 'import sys; sys.exit(10)'])
        self.assertEqual(popen.wait(), 10)

    def test_wait(self):
        popen = subprocess.Popen([sys.executable, '-c', 'import sys; sys.exit(11)'])
        gevent.wait([popen])
        self.assertEqual(popen.poll(), 11)

    def test_child_exception(self):
        try:
            subprocess.Popen(['*']).wait()
        except OSError as ex:
            assert ex.errno == 2, ex
        else:
            raise AssertionError('Expected OSError: [Errno 2] No such file or directory')

    def test_leak(self):
        num_before = greentest.get_number_open_files()
        p = subprocess.Popen([sys.executable, "-c", "print()"],
                             stdout=subprocess.PIPE)
        p.wait()
        del p
        if PYPY:
            gc.collect()
            gc.collect()
        num_after = greentest.get_number_open_files()
        self.assertEqual(num_before, num_after)

    def test_communicate(self):
        p = subprocess.Popen([sys.executable, "-c",
                              'import sys,os;'
                              'sys.stderr.write("pineapple");'
                              'sys.stdout.write(sys.stdin.read())'],
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        (stdout, stderr) = p.communicate(b"banana")
        self.assertEqual(stdout, b"banana")
        if sys.executable.endswith('-dbg'):
            assert stderr.startswith(b'pineapple')
        else:
            self.assertEqual(stderr, b"pineapple")

    def test_universal1(self):
        p = subprocess.Popen([sys.executable, "-c",
                              'import sys,os;' + SETBINARY +
                              'sys.stdout.write("line1\\n");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line2\\r");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line3\\r\\n");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line4\\r");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("\\nline5");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("\\nline6");'],
                             stdout=subprocess.PIPE,
                             universal_newlines=1,
                             bufsize=1)
        try:
            stdout = p.stdout.read()
            if PY3 and isinstance(stdout, bytes):
                # OS X gives us binary back from stdout.read, but linux (travis ci)
                # gives us text...text is correct because we're in universal newline
                # mode
                stdout = stdout.decode('ascii')
            if python_universal_newlines:
                # Interpreter with universal newline support
                self.assertEqual(stdout,
                                 "line1\nline2\nline3\nline4\nline5\nline6")
            else:
                # Interpreter without universal newline support
                self.assertEqual(stdout,
                                 "line1\nline2\rline3\r\nline4\r\nline5\nline6")
        finally:
            p.stdout.close()

    def test_universal2(self):
        p = subprocess.Popen([sys.executable, "-c",
                              'import sys,os;' + SETBINARY +
                              'sys.stdout.write("line1\\n");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line2\\r");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line3\\r\\n");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("line4\\r\\nline5");'
                              'sys.stdout.flush();'
                              'sys.stdout.write("\\nline6");'],
                             stdout=subprocess.PIPE,
                             universal_newlines=1,
                             bufsize=1)
        try:
            stdout = p.stdout.read()
            if python_universal_newlines:
                # Interpreter with universal newline support
                self.assertEqual(stdout,
                                 "line1\nline2\nline3\nline4\nline5\nline6")
            else:
                # Interpreter without universal newline support
                self.assertEqual(stdout,
                                 "line1\nline2\rline3\r\nline4\r\nline5\nline6")
        finally:
            p.stdout.close()

    if sys.platform != 'win32':

        def test_nonblock_removed(self):
            # see issue #134
            r, w = os.pipe()
            stdin = subprocess.FileObject(r)
            p = subprocess.Popen(['grep', 'text'], stdin=stdin)
            try:
                # Closing one half of the pipe causes Python 3 on OS X to terminate the
                # child process; it exits with code 1 and the assert that p.poll is None
                # fails. Removing the close lets it pass under both Python 3 and 2.7.
                # If subprocess.Popen._remove_nonblock_flag is changed to a noop, then
                # the test fails (as expected) even with the close removed
                #os.close(w)
                time.sleep(0.1)
                self.assertEqual(p.poll(), None)
            finally:
                if p.poll() is None:
                    p.kill()
                stdin.close()
                os.close(w)

    def test_issue148(self):
        for i in range(7):
            try:
                subprocess.Popen('this_name_must_not_exist')
            except OSError as ex:
                if ex.errno != errno.ENOENT:
                    raise
            else:
                raise AssertionError('must fail with ENOENT')

    def test_check_output_keyword_error(self):
        try:
            subprocess.check_output([sys.executable, '-c', 'import sys; sys.exit(44)'])
        except subprocess.CalledProcessError as e:
            self.assertEqual(e.returncode, 44)
        else:
            raise AssertionError('must fail with CalledProcessError')

    def test_popen_bufsize(self):
        # Test that subprocess has unbuffered output by default
        # (as the vanilla subprocess module)
        if PY3:
            # The default changed under python 3.
            return
        p = subprocess.Popen([sys.executable, '-u', '-c',
                              'import sys; sys.stdout.write(sys.stdin.readline())'],
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        p.stdin.write(b'foobar\n')
        r = p.stdout.readline()
        self.assertEqual(r, b'foobar\n')


if __name__ == '__main__':
    greentest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import six
import re
import greentest
import socket
from time import time
import gevent
import gevent.socket as gevent_socket
from util import log
from six import xrange


resolver = gevent.get_hub().resolver
log('Resolver: %s', resolver)

if getattr(resolver, 'pool', None) is not None:
    resolver.pool.size = 1


assert gevent_socket.gaierror is socket.gaierror
assert gevent_socket.error is socket.error

DEBUG = False


def _run(function, *args):
    try:
        result = function(*args)
        assert not isinstance(result, BaseException), repr(result)
        return result
    except Exception as ex:
        return ex


def format_call(function, args):
    args = repr(args)
    if args.endswith(',)'):
        args = args[:-2] + ')'
    try:
        module = function.__module__.replace('gevent.socket', 'gevent').replace('_socket', 'stdlib')
        name = function.__name__
        return '%s:%s%s' % (module, name, args)
    except AttributeError:
        return function + args


def log_fresult(result, seconds):
    if isinstance(result, Exception):
        msg = '  -=>  raised %r' % (result, )
    else:
        msg = '  -=>  returned %r' % (result, )
    time_ms = ' %.2fms' % (seconds * 1000.0, )
    space = 80 - len(msg) - len(time_ms)
    if space > 0:
        space = ' ' * space
    else:
        space = ''
    log(msg + space + time_ms)


def run(function, *args):
    if DEBUG:
        log(format_call(function, args))
    delta = time()
    result = _run(function, *args)
    delta = time() - delta
    if DEBUG:
        log_fresult(result, delta)
    return result, delta


def log_call(result, time, function, *args):
    log(format_call(function, args))
    log_fresult(result, time)


def compare_relaxed(a, b):
    """
    >>> compare_relaxed('2a00:1450:400f:801::1010', '2a00:1450:400f:800::1011')
    True

    >>> compare_relaxed('2a00:1450:400f:801::1010', '2aXX:1450:400f:900::1011')
    False

    >>> compare_relaxed('2a00:1450:4016:800::1013', '2a00:1450:4008:c01::93')
    True

    >>> compare_relaxed('2001:470::e852:4a38:9d7f:0', '2001:470:6d00:1c:1::d00')
    True

    >>> compare_relaxed('2001:470:4147:4943:6161:6161:2e74:6573', '2001:470::')
    True

    >>> compare_relaxed('2607:f8b0:6708:24af:1fd:700:60d4:4af', '2607:f8b0:2d00::f000:0')
    True

    >>> compare_relaxed('a.google.com', 'b.google.com')
    True

    >>> compare_relaxed('a.google.com', 'a.gevent.org')
    False
    """
    # IPv6 address from different requests might be different
    a_segments = a.count(':')
    b_segments = b.count(':')
    if a_segments and b_segments:
        if a_segments == b_segments and a_segments in (4, 5, 6, 7):
            return True
        if a.rstrip(':').startswith(b.rstrip(':')) or b.rstrip(':').startswith(a.rstrip(':')):
            return True
        if a_segments >= 2 and b_segments >= 2 and a.split(':')[:2] == b.split(':')[:2]:
            return True

    return a.split('.', 1)[-1] == b.split('.', 1)[-1]


def contains_5tuples(lst):
    for item in lst:
        if not (isinstance(item, tuple) and len(item) == 5):
            return False
    return True


def relaxed_is_equal(a, b):
    """
    >>> relaxed_is_equal([(10, 1, 6, '', ('2a00:1450:400f:801::1010', 80, 0, 0))], [(10, 1, 6, '', ('2a00:1450:400f:800::1011', 80, 0, 0))])
    True

    >>> relaxed_is_equal([1, '2'], (1, '2'))
    False

    >>> relaxed_is_equal([1, '2'], [1, '2'])
    True

    >>> relaxed_is_equal(('wi-in-x93.1e100.net', 'http'), ('we-in-x68.1e100.net', 'http'))
    True
    """
    if type(a) is not type(b):
        return False
    if a == b:
        return True
    if isinstance(a, six.string_types):
        return compare_relaxed(a, b)
    if len(a) != len(b):
        return False
    if contains_5tuples(a) and contains_5tuples(b):
        # getaddrinfo results
        a = sorted(a)
        b = sorted(b)
    return all(relaxed_is_equal(x, y) for (x, y) in zip(a, b))


def add(klass, hostname, name=None):

    call = callable(hostname)

    if name is None:
        if call:
            name = hostname.__name__
        else:
            name = re.sub('[^\w]+', '_', repr(hostname))
        assert name, repr(hostname)

    def test1(self):
        x = hostname() if call else hostname
        self._test('getaddrinfo', x, 'http')
    test1.__name__ = 'test_%s_getaddrinfo' % name
    setattr(klass, test1.__name__, test1)

    def test2(self):
        x = hostname() if call else hostname
        ipaddr = self._test('gethostbyname', x)
        if not isinstance(ipaddr, Exception):
            self._test('gethostbyaddr', ipaddr)
    test2.__name__ = 'test_%s_gethostbyname' % name
    setattr(klass, test2.__name__, test2)

    def test3(self):
        x = hostname() if call else hostname
        self._test('gethostbyname_ex', x)
    test3.__name__ = 'test_%s_gethostbyname_ex' % name
    setattr(klass, test3.__name__, test3)

    def test4(self):
        x = hostname() if call else hostname
        self._test('gethostbyaddr', x)
    test4.__name__ = 'test_%s_gethostbyaddr' % name
    setattr(klass, test4.__name__, test4)

    def test5(self):
        x = hostname() if call else hostname
        self._test('getnameinfo', (x, 80), 0)
    test5.__name__ = 'test_%s_getnameinfo' % name
    setattr(klass, test5.__name__, test5)


class TestCase(greentest.TestCase):

    __timeout__ = 30
    switch_expected = None

    def should_log_results(self, result1, result2):
        if isinstance(result1, BaseException) and isinstance(result2, BaseException):
            return type(result1) is not type(result2)
        return repr(result1) != repr(result2)

    def _test(self, func, *args):
        gevent_func = getattr(gevent_socket, func)
        real_func = getattr(socket, func)
        real_result, time_real = run(real_func, *args)
        gevent_result, time_gevent = run(gevent_func, *args)
        if not DEBUG and self.should_log_results(real_result, gevent_result):
            log('')
            log_call(real_result, time_real, real_func, *args)
            log_call(gevent_result, time_gevent, gevent_func, *args)
        self.assertEqualResults(real_result, gevent_result, func, args)

        if time_gevent > time_real + 0.01 and time_gevent > 0.02:
            msg = 'gevent:%s%s took %dms versus %dms stdlib' % (func, args, time_gevent * 1000.0, time_real * 1000.0)

            if time_gevent > time_real + 1:
                word = 'VERY'
            else:
                word = 'quite'

            log('\nWARNING: %s slow: %s', word, msg)

        return gevent_result

    def _normalize_result(self, result):
        return result

    def assertEqualResults(self, real_result, gevent_result, func, args):
        errors = (socket.gaierror, socket.herror, TypeError)
        if isinstance(real_result, errors) and isinstance(gevent_result, errors):
            if type(real_result) is not type(gevent_result):
                log('WARNING: error type mismatch: %r (gevent) != %r (stdlib)', gevent_result, real_result)
            return

        real_result = self._normalize_result(real_result)
        gevent_result = self._normalize_result(gevent_result)

        real_result_repr = repr(real_result)
        gevent_result_repr = repr(gevent_result)
        if real_result_repr == gevent_result_repr:
            return
        if relaxed_is_equal(gevent_result, real_result):
            return

        # From 2.7 on, assertEqual does a better job highlighting the results than we would
        # because it calls assertSequenceEqual, which highlights the exact
        # difference in the tuple
        msg = format_call(func, args)
        self.assertEqual((msg, gevent_result), (msg, real_result))


class TestTypeError(TestCase):
    pass

add(TestTypeError, None)
add(TestTypeError, 25)


class TestHostname(TestCase):
    pass

add(TestHostname, socket.gethostname)


class TestLocalhost(TestCase):
    # certain tests in test_patched_socket.py only work if getaddrinfo('localhost') does not switch
    # (e.g. NetworkConnectionAttributesTest.testSourceAddress)
    pass
    #switch_expected = False

add(TestLocalhost, 'localhost')
add(TestLocalhost, 'ip6-localhost')


class TestNonexistent(TestCase):
    pass

add(TestNonexistent, 'nonexistentxxxyyy')


class Test1234(TestCase):
    pass

add(Test1234, '1.2.3.4')


class Test127001(TestCase):
    pass

add(Test127001, '127.0.0.1')


class TestBroadcast(TestCase):
    switch_expected = False


add(TestBroadcast, '<broadcast>')


class TestEtcHosts(TestCase):
    pass

try:
    etc_hosts = open('/etc/hosts').read()
except IOError:
    etc_hosts = ''

for ip, host in re.findall(r'^\s*(\d+\.\d+\.\d+\.\d+)\s+([^\s]+)', etc_hosts, re.M)[:10]:
    add(TestEtcHosts, host)
    add(TestEtcHosts, ip)
    del host, ip


class TestGeventOrg(TestCase):

    HOSTNAME = 'www.gevent.org'

# For this test to work correctly, it needs to resolve to
# an address with a single A record; round-robin DNS and multiple A records
# may mess it up (subsequent requests---and we always make two---may return
# unequal results). We used to use gevent.org, but that now has multiple A records;
# trying www.gevent.org which is a CNAME to readthedocs.org.
add(TestGeventOrg, TestGeventOrg.HOSTNAME)


class TestFamily(TestCase):

    @classmethod
    def getresult(cls):
        if not hasattr(cls, '_result'):
            cls._result = getattr(socket, 'getaddrinfo')(TestGeventOrg.HOSTNAME, None)
        return cls._result

    def assert_error(self, error, function, *args):
        try:
            result = function(*args)
            raise AssertionError('%s: Expected to raise %s, instead returned %r' % (function, error, result))
        except Exception as ex:
            if isinstance(error, six.string_types):
                repr_error = error
            else:
                repr_error = repr(error)
                if type(ex) is not type(error):
                    raise
            if repr(ex) == repr_error:
                return
            raise

    def test_inet(self):
        self.assertEqual(gevent_socket.getaddrinfo(TestGeventOrg.HOSTNAME, None, socket.AF_INET), self.getresult())

    def test_unspec(self):
        self.assertEqual(gevent_socket.getaddrinfo(TestGeventOrg.HOSTNAME, None, socket.AF_UNSPEC), self.getresult())

    def test_badvalue(self):
        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, 255)
        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, 255000)
        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, None, -1)

    def test_badtype(self):
        self._test('getaddrinfo', TestGeventOrg.HOSTNAME, 'x')


class Test_getaddrinfo(TestCase):

    def _test_getaddrinfo(self, *args):
        self._test('getaddrinfo', *args)

    def test_80(self):
        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 80)

    def test_int_string(self):
        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, '80')

    def test_0(self):
        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 0)

    def test_http(self):
        self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 'http')

    def test_notexistent_tld(self):
        self._test_getaddrinfo('myhost.mytld', 53)

    def test_notexistent_dot_com(self):
        self._test_getaddrinfo('sdfsdfgu5e66098032453245wfdggd.com', 80)

    def test1(self):
        return self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 52, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, 0)

    def test2(self):
        return self._test_getaddrinfo(TestGeventOrg.HOSTNAME, 53, socket.AF_INET, socket.SOCK_DGRAM, 17)

    def test3(self):
        return self._test_getaddrinfo('google.com', 'http', socket.AF_INET6)


class TestInternational(TestCase):
    pass

add(TestInternational, u'президент.рф', 'russian')
add(TestInternational, u'президент.рф'.encode('idna'), 'idna')


class TestInterrupted_gethostbyname(greentest.GenericWaitTestCase):

    def wait(self, timeout):
        with gevent.Timeout(timeout, False):
            for index in xrange(1000000):
                try:
                    gevent_socket.gethostbyname('www.x%s.com' % index)
                except socket.error:
                    pass
            raise AssertionError('Timeout was not raised')

    def cleanup(self):
        gevent.get_hub().threadpool.join()


# class TestInterrupted_getaddrinfo(greentest.GenericWaitTestCase):
#
#     def wait(self, timeout):
#         with gevent.Timeout(timeout, False):
#             for index in range(1000):
#                 try:
#                     gevent_socket.getaddrinfo('www.a%s.com' % index, 'http')
#                 except socket.gaierror:
#                     pass


class TestBadName(TestCase):
    pass

add(TestBadName, 'xxxxxxxxxxxx')


class TestBadIP(TestCase):
    pass

add(TestBadIP, '1.2.3.400')


class Test_getnameinfo_127001(TestCase):

    def test(self):
        assert gevent_socket.getnameinfo is not socket.getnameinfo
        self._test('getnameinfo', ('127.0.0.1', 80), 0)

    def test_DGRAM(self):
        self._test('getnameinfo', ('127.0.0.1', 779), 0)
        self._test('getnameinfo', ('127.0.0.1', 779), socket.NI_DGRAM)

    def test_NOFQDN(self):
        # I get ('localhost', 'www') with _socket but ('localhost.localdomain', 'www') with gevent.socket
        self._test('getnameinfo', ('127.0.0.1', 80), socket.NI_NOFQDN)

    def test_NAMEREQD(self):
        self._test('getnameinfo', ('127.0.0.1', 80), socket.NI_NAMEREQD)


class Test_getnameinfo_geventorg(TestCase):

    def test_NUMERICHOST(self):
        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), 0)
        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), socket.NI_NUMERICHOST)

    def test_NUMERICSERV(self):
        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), socket.NI_NUMERICSERV)

    def test_domain1(self):
        self._test('getnameinfo', (TestGeventOrg.HOSTNAME, 80), 0)

    def test_domain2(self):
        self._test('getnameinfo', ('www.gevent.org', 80), 0)

    def test_port_zero(self):
        self._test('getnameinfo', ('www.gevent.org', 0), 0)


class Test_getnameinfo_fail(TestCase):

    def test_port_string(self):
        self._test('getnameinfo', ('www.gevent.org', 'http'), 0)

    def test_bad_flags(self):
        self._test('getnameinfo', ('127.0.0.1', 80), 55555555)


class TestInvalidPort(TestCase):

    def test1(self):
        self._test('getnameinfo', ('www.gevent.org', -1), 0)

    def test2(self):
        self._test('getnameinfo', ('www.gevent.org', None), 0)

    def test3(self):
        self._test('getnameinfo', ('www.gevent.org', 'x'), 0)

    def test4(self):
        self._test('getnameinfo', ('www.gevent.org', 65536), 0)


if __name__ == '__main__':
    greentest.main()
